@@ -1,7 +1,7 @@
 /*
  *  add.c
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -53,6 +53,7 @@ int SYMEXPORT alpm_add_pkg(alpm_handle_t *handle, alpm_pkg_t *pkg)
 	const char *pkgname, *pkgver;
 	alpm_trans_t *trans;
 	alpm_pkg_t *local;
+	alpm_pkg_t *dup;
 
 	/* Sanity checks */
 	CHECK_HANDLE(handle, return -1);
@@ -70,12 +71,16 @@ int SYMEXPORT alpm_add_pkg(alpm_handle_t *handle, alpm_pkg_t *pkg)
 
 	_alpm_log(handle, ALPM_LOG_DEBUG, "adding package '%s'\n", pkgname);
 
-	if(alpm_pkg_find(trans->add, pkgname)) {
+	if((dup = alpm_pkg_find(trans->add, pkgname))) {
+		if(dup == pkg) {
+			_alpm_log(handle, ALPM_LOG_DEBUG, "skipping duplicate target: %s\n", pkgname);
+			return 0;
+		}
+		/* error for separate packages with the same name */
 		RET_ERR(handle, ALPM_ERR_TRANS_DUP_TARGET, -1);
 	}
 
-	local = _alpm_db_get_pkgfromcache(handle->db_local, pkgname);
-	if(local) {
+	if((local = _alpm_db_get_pkgfromcache(handle->db_local, pkgname))) {
 		const char *localpkgname = local->name;
 		const char *localpkgver = local->version;
 		int cmp = _alpm_pkg_compare_versions(pkg, local);
@@ -427,7 +432,7 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 	ASSERT(trans != NULL, return -1);
 
 	/* see if this is an upgrade. if so, remove the old package first */
-	if((oldpkg = newpkg->oldpkg)) {
+	if(_alpm_db_get_pkgfromcache(db, newpkg->name) && (oldpkg = newpkg->oldpkg)) {
 		int cmp = _alpm_pkg_compare_versions(newpkg, oldpkg);
 		if(cmp < 0) {
 			log_msg = "downgrading";
@@ -694,5 +699,3 @@ int _alpm_upgrade_packages(alpm_handle_t *handle)
 
 	return ret;
 }
-
-/* vim: set noet: */
