@@ -1,7 +1,7 @@
 /*
  * alpm.h
  *
- *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
@@ -20,8 +20,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-#ifndef _ALPM_H
-#define _ALPM_H
+#ifndef ALPM_H
+#define ALPM_H
 
 #ifdef __cplusplus
 extern "C" {
@@ -41,6 +41,94 @@ extern "C" {
  * Arch Linux Package Management library
  */
 
+/*
+ * Opaque Structures
+ */
+typedef struct __alpm_handle_t alpm_handle_t;
+typedef struct __alpm_db_t alpm_db_t;
+typedef struct __alpm_pkg_t alpm_pkg_t;
+typedef struct __alpm_trans_t alpm_trans_t;
+
+/** @addtogroup alpm_api_errors Error Codes
+ * @{
+ */
+typedef enum _alpm_errno_t {
+	ALPM_ERR_OK = 0,
+	ALPM_ERR_MEMORY,
+	ALPM_ERR_SYSTEM,
+	ALPM_ERR_BADPERMS,
+	ALPM_ERR_NOT_A_FILE,
+	ALPM_ERR_NOT_A_DIR,
+	ALPM_ERR_WRONG_ARGS,
+	ALPM_ERR_DISK_SPACE,
+	/* Interface */
+	ALPM_ERR_HANDLE_NULL,
+	ALPM_ERR_HANDLE_NOT_NULL,
+	ALPM_ERR_HANDLE_LOCK,
+	/* Databases */
+	ALPM_ERR_DB_OPEN,
+	ALPM_ERR_DB_CREATE,
+	ALPM_ERR_DB_NULL,
+	ALPM_ERR_DB_NOT_NULL,
+	ALPM_ERR_DB_NOT_FOUND,
+	ALPM_ERR_DB_INVALID,
+	ALPM_ERR_DB_INVALID_SIG,
+	ALPM_ERR_DB_VERSION,
+	ALPM_ERR_DB_WRITE,
+	ALPM_ERR_DB_REMOVE,
+	/* Servers */
+	ALPM_ERR_SERVER_BAD_URL,
+	ALPM_ERR_SERVER_NONE,
+	/* Transactions */
+	ALPM_ERR_TRANS_NOT_NULL,
+	ALPM_ERR_TRANS_NULL,
+	ALPM_ERR_TRANS_DUP_TARGET,
+	ALPM_ERR_TRANS_NOT_INITIALIZED,
+	ALPM_ERR_TRANS_NOT_PREPARED,
+	ALPM_ERR_TRANS_ABORT,
+	ALPM_ERR_TRANS_TYPE,
+	ALPM_ERR_TRANS_NOT_LOCKED,
+	ALPM_ERR_TRANS_HOOK_FAILED,
+	/* Packages */
+	ALPM_ERR_PKG_NOT_FOUND,
+	ALPM_ERR_PKG_IGNORED,
+	ALPM_ERR_PKG_INVALID,
+	ALPM_ERR_PKG_INVALID_CHECKSUM,
+	ALPM_ERR_PKG_INVALID_SIG,
+	ALPM_ERR_PKG_MISSING_SIG,
+	ALPM_ERR_PKG_OPEN,
+	ALPM_ERR_PKG_CANT_REMOVE,
+	ALPM_ERR_PKG_INVALID_NAME,
+	ALPM_ERR_PKG_INVALID_ARCH,
+	ALPM_ERR_PKG_REPO_NOT_FOUND,
+	/* Signatures */
+	ALPM_ERR_SIG_MISSING,
+	ALPM_ERR_SIG_INVALID,
+	/* Dependencies */
+	ALPM_ERR_UNSATISFIED_DEPS,
+	ALPM_ERR_CONFLICTING_DEPS,
+	ALPM_ERR_FILE_CONFLICTS,
+	/* Misc */
+	ALPM_ERR_RETRIEVE,
+	ALPM_ERR_INVALID_REGEX,
+	/* External library errors */
+	ALPM_ERR_LIBARCHIVE,
+	ALPM_ERR_LIBCURL,
+	ALPM_ERR_EXTERNAL_DOWNLOAD,
+	ALPM_ERR_GPGME,
+	/* Missing compile-time features */
+	ALPM_ERR_MISSING_CAPABILITY_SIGNATURES
+} alpm_errno_t;
+
+/** Returns the current error code from the handle. */
+alpm_errno_t alpm_errno(alpm_handle_t *handle);
+
+/** Returns the string corresponding to an error number. */
+const char *alpm_strerror(alpm_errno_t err);
+
+/* End of alpm_api_errors */
+/** @} */
+
 /** @addtogroup alpm_api Public API
  * The libalpm Public API
  * @{
@@ -79,17 +167,17 @@ typedef enum _alpm_pkgvalidation_t {
 
 /** Types of version constraints in dependency specs. */
 typedef enum _alpm_depmod_t {
-  /** No version constraint */
+	/** No version constraint */
 	ALPM_DEP_MOD_ANY = 1,
-  /** Test version equality (package=x.y.z) */
+	/** Test version equality (package=x.y.z) */
 	ALPM_DEP_MOD_EQ,
-  /** Test for at least a version (package>=x.y.z) */
+	/** Test for at least a version (package>=x.y.z) */
 	ALPM_DEP_MOD_GE,
-  /** Test for at most a version (package<=x.y.z) */
+	/** Test for at most a version (package<=x.y.z) */
 	ALPM_DEP_MOD_LE,
-  /** Test for greater than some version (package>x.y.z) */
+	/** Test for greater than some version (package>x.y.z) */
 	ALPM_DEP_MOD_GT,
-  /** Test for less than some version (package<x.y.z) */
+	/** Test for less than some version (package<x.y.z) */
 	ALPM_DEP_MOD_LT
 } alpm_depmod_t;
 
@@ -115,10 +203,7 @@ typedef enum _alpm_siglevel_t {
 	ALPM_SIG_DATABASE_MARGINAL_OK = (1 << 12),
 	ALPM_SIG_DATABASE_UNKNOWN_OK = (1 << 13),
 
-	ALPM_SIG_PACKAGE_SET = (1 << 27),
-	ALPM_SIG_PACKAGE_TRUST_SET = (1 << 28),
-
-	ALPM_SIG_USE_DEFAULT = (1 << 31)
+	ALPM_SIG_USE_DEFAULT = (1 << 30)
 } alpm_siglevel_t;
 
 /** PGP signature verification status return codes */
@@ -143,11 +228,6 @@ typedef enum _alpm_sigvalidity_t {
  * Structures
  */
 
-typedef struct __alpm_handle_t alpm_handle_t;
-typedef struct __alpm_db_t alpm_db_t;
-typedef struct __alpm_pkg_t alpm_pkg_t;
-typedef struct __alpm_trans_t alpm_trans_t;
-
 /** Dependency */
 typedef struct _alpm_depend_t {
 	char *name;
@@ -190,22 +270,6 @@ typedef struct _alpm_group_t {
 	alpm_list_t *packages;
 } alpm_group_t;
 
-/** Package upgrade delta */
-typedef struct _alpm_delta_t {
-	/** filename of the delta patch */
-	char *delta;
-	/** md5sum of the delta file */
-	char *delta_md5;
-	/** filename of the 'before' file */
-	char *from;
-	/** filename of the 'after' file */
-	char *to;
-	/** filesize of the delta file */
-	off_t delta_size;
-	/** download filesize of the delta file */
-	off_t download_size;
-} alpm_delta_t;
-
 /** File in a package */
 typedef struct _alpm_file_t {
 	char *name;
@@ -217,7 +281,6 @@ typedef struct _alpm_file_t {
 typedef struct _alpm_filelist_t {
 	size_t count;
 	alpm_file_t *files;
-	char **resolved_path;
 } alpm_filelist_t;
 
 /** Local package or package file backup entry */
@@ -251,13 +314,23 @@ typedef struct _alpm_sigresult_t {
 
 /**
  * Signature list. Contains the number of signatures found and a pointer to an
- * array of results.  The array is of size count.
+ * array of results. The array is of size count.
  */
 typedef struct _alpm_siglist_t {
 	size_t count;
 	alpm_sigresult_t *results;
 } alpm_siglist_t;
 
+
+/*
+ * Hooks
+ */
+
+typedef enum _alpm_hook_when_t {
+	ALPM_HOOK_PRE_TRANSACTION = 1,
+	ALPM_HOOK_POST_TRANSACTION
+} alpm_hook_when_t;
+
 /*
  * Logging facilities
  */
@@ -276,10 +349,9 @@ int alpm_logaction(alpm_handle_t *handle, const char *prefix,
 		const char *fmt, ...) __attribute__((format(printf, 3, 4)));
 
 /**
- * Events.
- * NULL parameters are passed to in all events unless specified otherwise.
+ * Type of events.
  */
-typedef enum _alpm_event_t {
+typedef enum _alpm_event_type_t {
 	/** Dependencies will be computed for a package. */
 	ALPM_EVENT_CHECKDEPS_START = 1,
 	/** Dependencies were computed for a package. */
@@ -296,49 +368,16 @@ typedef enum _alpm_event_t {
 	ALPM_EVENT_INTERCONFLICTS_START,
 	/** Inter-conflicts were checked for target package. */
 	ALPM_EVENT_INTERCONFLICTS_DONE,
-	/** Package will be installed.
-	 * A pointer to the target package is passed to the callback.
-	 */
-	ALPM_EVENT_ADD_START,
-	/** Package was installed.
-	 * A pointer to the new package is passed to the callback.
-	 */
-	ALPM_EVENT_ADD_DONE,
-	/** Package will be removed.
-	 * A pointer to the target package is passed to the callback.
-	 */
-	ALPM_EVENT_REMOVE_START,
-	/** Package was removed.
-	 * A pointer to the removed package is passed to the callback.
-	 */
-	ALPM_EVENT_REMOVE_DONE,
-	/** Package will be upgraded.
-	 * A pointer to the upgraded package is passed to the callback.
-	 */
-	ALPM_EVENT_UPGRADE_START,
-	/** Package was upgraded.
-	 * A pointer to the new package, and a pointer to the old package is passed
-	 * to the callback, respectively.
-	 */
-	ALPM_EVENT_UPGRADE_DONE,
-	/** Package will be downgraded.
-	 * A pointer to the downgraded package is passed to the callback.
-	 */
-	ALPM_EVENT_DOWNGRADE_START,
-	/** Package was downgraded.
-	 * A pointer to the new package, and a pointer to the old package is passed
-	 * to the callback, respectively.
-	 */
-	ALPM_EVENT_DOWNGRADE_DONE,
-	/** Package will be reinstalled.
-	 * A pointer to the reinstalled package is passed to the callback.
-	 */
-	ALPM_EVENT_REINSTALL_START,
-	/** Package was reinstalled.
-	 * A pointer to the new package, and a pointer to the old package is passed
-	 * to the callback, respectively.
-	 */
-	ALPM_EVENT_REINSTALL_DONE,
+	/** Processing the package transaction is starting. */
+	ALPM_EVENT_TRANSACTION_START,
+	/** Processing the package transaction is finished. */
+	ALPM_EVENT_TRANSACTION_DONE,
+	/** Package will be installed/upgraded/downgraded/re-installed/removed; See
+	 * alpm_event_package_operation_t for arguments. */
+	ALPM_EVENT_PACKAGE_OPERATION_START,
+	/** Package was installed/upgraded/downgraded/re-installed/removed; See
+	 * alpm_event_package_operation_t for arguments. */
+	ALPM_EVENT_PACKAGE_OPERATION_DONE,
 	/** Target package's integrity will be checked. */
 	ALPM_EVENT_INTEGRITY_START,
 	/** Target package's integrity was checked. */
@@ -347,39 +386,33 @@ typedef enum _alpm_event_t {
 	ALPM_EVENT_LOAD_START,
 	/** Target package is finished loading. */
 	ALPM_EVENT_LOAD_DONE,
-	/** Target delta's integrity will be checked. */
-	ALPM_EVENT_DELTA_INTEGRITY_START,
-	/** Target delta's integrity was checked. */
-	ALPM_EVENT_DELTA_INTEGRITY_DONE,
-	/** Deltas will be applied to packages. */
-	ALPM_EVENT_DELTA_PATCHES_START,
-	/** Deltas were applied to packages. */
-	ALPM_EVENT_DELTA_PATCHES_DONE,
-	/** Delta patch will be applied to target package.
-	 * The filename of the package and the filename of the patch is passed to the
-	 * callback.
-	 */
-	ALPM_EVENT_DELTA_PATCH_START,
-	/** Delta patch was applied to target package. */
-	ALPM_EVENT_DELTA_PATCH_DONE,
-	/** Delta patch failed to apply to target package. */
-	ALPM_EVENT_DELTA_PATCH_FAILED,
-	/** Scriptlet has printed information.
-	 * A line of text is passed to the callback.
-	 */
+	/** Scriptlet has printed information; See alpm_event_scriptlet_info_t for
+	 * arguments. */
 	ALPM_EVENT_SCRIPTLET_INFO,
-	/** Files will be downloaded from a repository.
-	 * The repository's tree name is passed to the callback.
-	 */
+	/** Files will be downloaded from a repository. */
 	ALPM_EVENT_RETRIEVE_START,
-	/** Disk space usage will be computed for a package */
+	/** Files were downloaded from a repository. */
+	ALPM_EVENT_RETRIEVE_DONE,
+	/** Not all files were successfully downloaded from a repository. */
+	ALPM_EVENT_RETRIEVE_FAILED,
+	/** A file will be downloaded from a repository; See alpm_event_pkgdownload_t
+	 * for arguments */
+	ALPM_EVENT_PKGDOWNLOAD_START,
+	/** A file was downloaded from a repository; See alpm_event_pkgdownload_t
+	 * for arguments */
+	ALPM_EVENT_PKGDOWNLOAD_DONE,
+	/** A file failed to be downloaded from a repository; See
+	 * alpm_event_pkgdownload_t for arguments */
+	ALPM_EVENT_PKGDOWNLOAD_FAILED,
+	/** Disk space usage will be computed for a package. */
 	ALPM_EVENT_DISKSPACE_START,
-	/** Disk space usage was computed for a package */
+	/** Disk space usage was computed for a package. */
 	ALPM_EVENT_DISKSPACE_DONE,
-	/** An optdepend for another package is being removed
-	 * The requiring package and its dependency are passed to the callback */
-	ALPM_EVENT_OPTDEP_REQUIRED,
-	/** A configured repository database is missing */
+	/** An optdepend for another package is being removed; See
+	 * alpm_event_optdep_removal_t for arguments. */
+	ALPM_EVENT_OPTDEP_REMOVAL,
+	/** A configured repository database is missing; See
+	 * alpm_event_database_missing_t for arguments. */
 	ALPM_EVENT_DATABASE_MISSING,
 	/** Checking keys used to create signatures are in keyring. */
 	ALPM_EVENT_KEYRING_START,
@@ -388,31 +421,259 @@ typedef enum _alpm_event_t {
 	/** Downloading missing keys into keyring. */
 	ALPM_EVENT_KEY_DOWNLOAD_START,
 	/** Key downloading is finished. */
-	ALPM_EVENT_KEY_DOWNLOAD_DONE
+	ALPM_EVENT_KEY_DOWNLOAD_DONE,
+	/** A .pacnew file was created; See alpm_event_pacnew_created_t for arguments. */
+	ALPM_EVENT_PACNEW_CREATED,
+	/** A .pacsave file was created; See alpm_event_pacsave_created_t for
+	 * arguments */
+	ALPM_EVENT_PACSAVE_CREATED,
+	/** Processing hooks will be started. */
+	ALPM_EVENT_HOOK_START,
+	/** Processing hooks is finished. */
+	ALPM_EVENT_HOOK_DONE,
+	/** A hook is starting */
+	ALPM_EVENT_HOOK_RUN_START,
+	/** A hook has finished running */
+	ALPM_EVENT_HOOK_RUN_DONE
+} alpm_event_type_t;
+
+typedef struct _alpm_event_any_t {
+	/** Type of event. */
+	alpm_event_type_t type;
+} alpm_event_any_t;
+
+typedef enum _alpm_package_operation_t {
+	/** Package (to be) installed. (No oldpkg) */
+	ALPM_PACKAGE_INSTALL = 1,
+	/** Package (to be) upgraded */
+	ALPM_PACKAGE_UPGRADE,
+	/** Package (to be) re-installed. */
+	ALPM_PACKAGE_REINSTALL,
+	/** Package (to be) downgraded. */
+	ALPM_PACKAGE_DOWNGRADE,
+	/** Package (to be) removed. (No newpkg) */
+	ALPM_PACKAGE_REMOVE
+} alpm_package_operation_t;
+
+typedef struct _alpm_event_package_operation_t {
+	/** Type of event. */
+	alpm_event_type_t type;
+	/** Type of operation. */
+	alpm_package_operation_t operation;
+	/** Old package. */
+	alpm_pkg_t *oldpkg;
+	/** New package. */
+	alpm_pkg_t *newpkg;
+} alpm_event_package_operation_t;
+
+typedef struct _alpm_event_optdep_removal_t {
+	/** Type of event. */
+	alpm_event_type_t type;
+	/** Package with the optdep. */
+	alpm_pkg_t *pkg;
+	/** Optdep being removed. */
+	alpm_depend_t *optdep;
+} alpm_event_optdep_removal_t;
+
+typedef struct _alpm_event_scriptlet_info_t {
+	/** Type of event. */
+	alpm_event_type_t type;
+	/** Line of scriptlet output. */
+	const char *line;
+} alpm_event_scriptlet_info_t;
+
+typedef struct _alpm_event_database_missing_t {
+	/** Type of event. */
+	alpm_event_type_t type;
+	/** Name of the database. */
+	const char *dbname;
+} alpm_event_database_missing_t;
+
+typedef struct _alpm_event_pkgdownload_t {
+	/** Type of event. */
+	alpm_event_type_t type;
+	/** Name of the file */
+	const char *file;
+} alpm_event_pkgdownload_t;
+
+typedef struct _alpm_event_pacnew_created_t {
+	/** Type of event. */
+	alpm_event_type_t type;
+	/** Whether the creation was result of a NoUpgrade or not */
+	int from_noupgrade;
+	/** Old package. */
+	alpm_pkg_t *oldpkg;
+	/** New Package. */
+	alpm_pkg_t *newpkg;
+	/** Filename of the file without the .pacnew suffix */
+	const char *file;
+} alpm_event_pacnew_created_t;
+
+typedef struct _alpm_event_pacsave_created_t {
+	/** Type of event. */
+	alpm_event_type_t type;
+	/** Old package. */
+	alpm_pkg_t *oldpkg;
+	/** Filename of the file without the .pacsave suffix. */
+	const char *file;
+} alpm_event_pacsave_created_t;
+
+typedef struct _alpm_event_hook_t {
+	/** Type of event.*/
+	alpm_event_type_t type;
+	/** Type of hooks. */
+	alpm_hook_when_t when;
+} alpm_event_hook_t;
+
+typedef struct _alpm_event_hook_run_t {
+	/** Type of event.*/
+	alpm_event_type_t type;
+	/** Name of hook */
+	const char *name;
+	/** Description of hook to be outputted */
+	const char *desc;
+	/** position of hook being run */
+	size_t position;
+	/** total hooks being run */
+	size_t total;
+} alpm_event_hook_run_t;
+
+/** Events.
+ * This is an union passed to the callback, that allows the frontend to know
+ * which type of event was triggered (via type). It is then possible to
+ * typecast the pointer to the right structure, or use the union field, in order
+ * to access event-specific data. */
+typedef union _alpm_event_t {
+	alpm_event_type_t type;
+	alpm_event_any_t any;
+	alpm_event_package_operation_t package_operation;
+	alpm_event_optdep_removal_t optdep_removal;
+	alpm_event_scriptlet_info_t scriptlet_info;
+	alpm_event_database_missing_t database_missing;
+	alpm_event_pkgdownload_t pkgdownload;
+	alpm_event_pacnew_created_t pacnew_created;
+	alpm_event_pacsave_created_t pacsave_created;
+	alpm_event_hook_t hook;
+	alpm_event_hook_run_t hook_run;
 } alpm_event_t;
 
-/** Event callback */
-typedef void (*alpm_cb_event)(alpm_event_t, void *, void *);
+/** Event callback. */
+typedef void (*alpm_cb_event)(alpm_event_t *);
 
 /**
- * Questions.
+ * Type of questions.
  * Unlike the events or progress enumerations, this enum has bitmask values
  * so a frontend can use a bitmask map to supply preselected answers to the
  * different types of questions.
  */
-typedef enum _alpm_question_t {
-	ALPM_QUESTION_INSTALL_IGNOREPKG = 1,
+typedef enum _alpm_question_type_t {
+	ALPM_QUESTION_INSTALL_IGNOREPKG = (1 << 0),
 	ALPM_QUESTION_REPLACE_PKG = (1 << 1),
 	ALPM_QUESTION_CONFLICT_PKG = (1 << 2),
 	ALPM_QUESTION_CORRUPTED_PKG = (1 << 3),
-	ALPM_QUESTION_LOCAL_NEWER = (1 << 4),
-	ALPM_QUESTION_REMOVE_PKGS = (1 << 5),
-	ALPM_QUESTION_SELECT_PROVIDER = (1 << 6),
-	ALPM_QUESTION_IMPORT_KEY = (1 << 7)
+	ALPM_QUESTION_REMOVE_PKGS = (1 << 4),
+	ALPM_QUESTION_SELECT_PROVIDER = (1 << 5),
+	ALPM_QUESTION_IMPORT_KEY = (1 << 6)
+} alpm_question_type_t;
+
+typedef struct _alpm_question_any_t {
+	/** Type of question. */
+	alpm_question_type_t type;
+	/** Answer. */
+	int answer;
+} alpm_question_any_t;
+
+typedef struct _alpm_question_install_ignorepkg_t {
+	/** Type of question. */
+	alpm_question_type_t type;
+	/** Answer: whether or not to install pkg anyway. */
+	int install;
+	/* Package in IgnorePkg/IgnoreGroup. */
+	alpm_pkg_t *pkg;
+} alpm_question_install_ignorepkg_t;
+
+typedef struct _alpm_question_replace_t {
+	/** Type of question. */
+	alpm_question_type_t type;
+	/** Answer: whether or not to replace oldpkg with newpkg. */
+	int replace;
+	/* Package to be replaced. */
+	alpm_pkg_t *oldpkg;
+	/* Package to replace with. */
+	alpm_pkg_t *newpkg;
+	/* DB of newpkg */
+	alpm_db_t *newdb;
+} alpm_question_replace_t;
+
+typedef struct _alpm_question_conflict_t {
+	/** Type of question. */
+	alpm_question_type_t type;
+	/** Answer: whether or not to remove conflict->package2. */
+	int remove;
+	/** Conflict info. */
+	alpm_conflict_t *conflict;
+} alpm_question_conflict_t;
+
+typedef struct _alpm_question_corrupted_t {
+	/** Type of question. */
+	alpm_question_type_t type;
+	/** Answer: whether or not to remove filepath. */
+	int remove;
+	/** Filename to remove */
+	const char *filepath;
+	/** Error code indicating the reason for package invalidity */
+	alpm_errno_t reason;
+} alpm_question_corrupted_t;
+
+typedef struct _alpm_question_remove_pkgs_t {
+	/** Type of question. */
+	alpm_question_type_t type;
+	/** Answer: whether or not to skip packages. */
+	int skip;
+	/** List of alpm_pkg_t* with unresolved dependencies. */
+	alpm_list_t *packages;
+} alpm_question_remove_pkgs_t;
+
+typedef struct _alpm_question_select_provider_t {
+	/** Type of question. */
+	alpm_question_type_t type;
+	/** Answer: which provider to use (index from providers). */
+	int use_index;
+	/** List of alpm_pkg_t* as possible providers. */
+	alpm_list_t *providers;
+	/** What providers provide for. */
+	alpm_depend_t *depend;
+} alpm_question_select_provider_t;
+
+typedef struct _alpm_question_import_key_t {
+	/** Type of question. */
+	alpm_question_type_t type;
+	/** Answer: whether or not to import key. */
+	int import;
+	/** The key to import. */
+	alpm_pgpkey_t *key;
+} alpm_question_import_key_t;
+
+/**
+ * Questions.
+ * This is an union passed to the callback, that allows the frontend to know
+ * which type of question was triggered (via type). It is then possible to
+ * typecast the pointer to the right structure, or use the union field, in order
+ * to access question-specific data. */
+typedef union _alpm_question_t {
+	alpm_question_type_t type;
+	alpm_question_any_t any;
+	alpm_question_install_ignorepkg_t install_ignorepkg;
+	alpm_question_replace_t replace;
+	alpm_question_conflict_t conflict;
+	alpm_question_corrupted_t corrupted;
+	alpm_question_remove_pkgs_t remove_pkgs;
+	alpm_question_select_provider_t select_provider;
+	alpm_question_import_key_t import_key;
 } alpm_question_t;
 
 /** Question callback */
-typedef void (*alpm_cb_question)(alpm_question_t, void *, void *, void *, int *);
+typedef void (*alpm_cb_question)(alpm_question_t *);
 
 /** Progress */
 typedef enum _alpm_progress_t {
@@ -520,6 +781,20 @@ int alpm_option_add_cachedir(alpm_handle_t *handle, const char *cachedir);
 int alpm_option_remove_cachedir(alpm_handle_t *handle, const char *cachedir);
 /** @} */
 
+/** @name Accessors to the list of package hook directories.
+ * @{
+ */
+alpm_list_t *alpm_option_get_hookdirs(alpm_handle_t *handle);
+int alpm_option_set_hookdirs(alpm_handle_t *handle, alpm_list_t *hookdirs);
+int alpm_option_add_hookdir(alpm_handle_t *handle, const char *hookdir);
+int alpm_option_remove_hookdir(alpm_handle_t *handle, const char *hookdir);
+/** @} */
+
+alpm_list_t *alpm_option_get_overwrite_files(alpm_handle_t *handle);
+int alpm_option_set_overwrite_files(alpm_handle_t *handle, alpm_list_t *globs);
+int alpm_option_add_overwrite_file(alpm_handle_t *handle, const char *glob);
+int alpm_option_remove_overwrite_file(alpm_handle_t *handle, const char *glob);
+
 /** Returns the logfile name. */
 const char *alpm_option_get_logfile(alpm_handle_t *handle);
 /** Sets the logfile name. */
@@ -541,9 +816,10 @@ int alpm_option_set_usesyslog(alpm_handle_t *handle, int usesyslog);
  * @{
  */
 alpm_list_t *alpm_option_get_noupgrades(alpm_handle_t *handle);
-int alpm_option_add_noupgrade(alpm_handle_t *handle, const char *pkg);
+int alpm_option_add_noupgrade(alpm_handle_t *handle, const char *path);
 int alpm_option_set_noupgrades(alpm_handle_t *handle, alpm_list_t *noupgrade);
-int alpm_option_remove_noupgrade(alpm_handle_t *handle, const char *pkg);
+int alpm_option_remove_noupgrade(alpm_handle_t *handle, const char *path);
+int alpm_option_match_noupgrade(alpm_handle_t *handle, const char *path);
 /** @} */
 
 /** @name Accessors to the list of no-extract files.
@@ -553,9 +829,10 @@ int alpm_option_remove_noupgrade(alpm_handle_t *handle, const char *pkg);
  * @{
  */
 alpm_list_t *alpm_option_get_noextracts(alpm_handle_t *handle);
-int alpm_option_add_noextract(alpm_handle_t *handle, const char *pkg);
+int alpm_option_add_noextract(alpm_handle_t *handle, const char *path);
 int alpm_option_set_noextracts(alpm_handle_t *handle, alpm_list_t *noextract);
-int alpm_option_remove_noextract(alpm_handle_t *handle, const char *pkg);
+int alpm_option_remove_noextract(alpm_handle_t *handle, const char *path);
+int alpm_option_match_noextract(alpm_handle_t *handle, const char *path);
 /** @} */
 
 /** @name Accessors to the list of ignored packages.
@@ -580,25 +857,38 @@ int alpm_option_set_ignoregroups(alpm_handle_t *handle, alpm_list_t *ignoregrps)
 int alpm_option_remove_ignoregroup(alpm_handle_t *handle, const char *grp);
 /** @} */
 
+/** @name Accessors to the list of ignored dependencies.
+ * These functions modify the list of dependencies that
+ * should be ignored by a sysupgrade.
+ * @{
+ */
+alpm_list_t *alpm_option_get_assumeinstalled(alpm_handle_t *handle);
+int alpm_option_add_assumeinstalled(alpm_handle_t *handle, const alpm_depend_t *dep);
+int alpm_option_set_assumeinstalled(alpm_handle_t *handle, alpm_list_t *deps);
+int alpm_option_remove_assumeinstalled(alpm_handle_t *handle, const alpm_depend_t *dep);
+/** @} */
+
 /** Returns the targeted architecture. */
 const char *alpm_option_get_arch(alpm_handle_t *handle);
 /** Sets the targeted architecture. */
 int alpm_option_set_arch(alpm_handle_t *handle, const char *arch);
 
-double alpm_option_get_deltaratio(alpm_handle_t *handle);
-int alpm_option_set_deltaratio(alpm_handle_t *handle, double ratio);
-
 int alpm_option_get_checkspace(alpm_handle_t *handle);
 int alpm_option_set_checkspace(alpm_handle_t *handle, int checkspace);
 
-alpm_siglevel_t alpm_option_get_default_siglevel(alpm_handle_t *handle);
-int alpm_option_set_default_siglevel(alpm_handle_t *handle, alpm_siglevel_t level);
+const char *alpm_option_get_dbext(alpm_handle_t *handle);
+int alpm_option_set_dbext(alpm_handle_t *handle, const char *dbext);
 
-alpm_siglevel_t alpm_option_get_local_file_siglevel(alpm_handle_t *handle);
-int alpm_option_set_local_file_siglevel(alpm_handle_t *handle, alpm_siglevel_t level);
+int alpm_option_get_default_siglevel(alpm_handle_t *handle);
+int alpm_option_set_default_siglevel(alpm_handle_t *handle, int level);
 
-alpm_siglevel_t alpm_option_get_remote_file_siglevel(alpm_handle_t *handle);
-int alpm_option_set_remote_file_siglevel(alpm_handle_t *handle, alpm_siglevel_t level);
+int alpm_option_get_local_file_siglevel(alpm_handle_t *handle);
+int alpm_option_set_local_file_siglevel(alpm_handle_t *handle, int level);
+
+int alpm_option_get_remote_file_siglevel(alpm_handle_t *handle);
+int alpm_option_set_remote_file_siglevel(alpm_handle_t *handle, int level);
+
+int alpm_option_set_disable_dl_timeout(alpm_handle_t *handle, unsigned short disable_dl_timeout);
 
 /** @} */
 
@@ -631,7 +921,7 @@ alpm_list_t *alpm_get_syncdbs(alpm_handle_t *handle);
  * @return an alpm_db_t* on success (the value), NULL on error
  */
 alpm_db_t *alpm_register_syncdb(alpm_handle_t *handle, const char *treename,
-		alpm_siglevel_t level);
+		int level);
 
 /** Unregister all package databases.
  * @param handle the context handle
@@ -657,7 +947,7 @@ const char *alpm_db_get_name(const alpm_db_t *db);
  * @param db pointer to the package database
  * @return the signature verification level
  */
-alpm_siglevel_t alpm_db_get_siglevel(alpm_db_t *db);
+int alpm_db_get_siglevel(alpm_db_t *db);
 
 /** Check the validity of a database.
  * This is most useful for sync databases and verifying signature status.
@@ -711,6 +1001,28 @@ alpm_list_t *alpm_db_get_groupcache(alpm_db_t *db);
  */
 alpm_list_t *alpm_db_search(alpm_db_t *db, const alpm_list_t *needles);
 
+typedef enum _alpm_db_usage_t {
+	ALPM_DB_USAGE_SYNC = 1,
+	ALPM_DB_USAGE_SEARCH = (1 << 1),
+	ALPM_DB_USAGE_INSTALL = (1 << 2),
+	ALPM_DB_USAGE_UPGRADE = (1 << 3),
+	ALPM_DB_USAGE_ALL = (1 << 4) - 1,
+} alpm_db_usage_t;
+
+/** Sets the usage of a database.
+ * @param db pointer to the package database to set the status for
+ * @param usage a bitmask of alpm_db_usage_t values
+ * @return 0 on success, or -1 on error
+ */
+int alpm_db_set_usage(alpm_db_t *db, int usage);
+
+/** Gets the usage of a database.
+ * @param db pointer to the package database to get the status of
+ * @param usage pointer to an alpm_db_usage_t to store db's status
+ * @return 0 on success, or -1 on error
+ */
+int alpm_db_get_usage(alpm_db_t *db, int *usage);
+
 /** @} */
 
 /** @addtogroup alpm_api_packages Package Functions
@@ -733,7 +1045,7 @@ alpm_list_t *alpm_db_search(alpm_db_t *db, const alpm_list_t *needles);
  * @return 0 on success, -1 on error (pm_errno is set accordingly)
  */
 int alpm_pkg_load(alpm_handle_t *handle, const char *filename, int full,
-		alpm_siglevel_t level, alpm_pkg_t **pkg);
+		int level, alpm_pkg_t **pkg);
 
 /** Find a package in a list by name.
  * @param haystack a list of alpm_pkg_t
@@ -773,6 +1085,15 @@ alpm_list_t *alpm_pkg_compute_requiredby(alpm_pkg_t *pkg);
  */
 alpm_list_t *alpm_pkg_compute_optionalfor(alpm_pkg_t *pkg);
 
+/** Test if a package should be ignored.
+ * Checks if the package is ignored via IgnorePkg, or if the package is
+ * in a group ignored via IgnoreGroup.
+ * @param handle the context handle
+ * @param pkg the package to test
+ * @return 1 if the package should be ignored, 0 otherwise
+ */
+int alpm_pkg_should_ignore(alpm_handle_t *handle, alpm_pkg_t *pkg);
+
 /** @name Package Property Accessors
  * Any pointer returned by these functions points to internal structures
  * allocated by libalpm. They should not be freed nor modified in any
@@ -786,6 +1107,12 @@ alpm_list_t *alpm_pkg_compute_optionalfor(alpm_pkg_t *pkg);
  */
 const char *alpm_pkg_get_filename(alpm_pkg_t *pkg);
 
+/** Returns the package base name.
+ * @param pkg a pointer to package
+ * @return a reference to an internal string
+ */
+const char *alpm_pkg_get_base(alpm_pkg_t *pkg);
+
 /** Returns the package name.
  * @param pkg a pointer to package
  * @return a reference to an internal string
@@ -898,6 +1225,18 @@ alpm_list_t *alpm_pkg_get_depends(alpm_pkg_t *pkg);
  */
 alpm_list_t *alpm_pkg_get_optdepends(alpm_pkg_t *pkg);
 
+/** Returns a list of package check dependencies
+ * @param pkg a pointer to package
+ * @return a reference to an internal list of alpm_depend_t structures.
+ */
+alpm_list_t *alpm_pkg_get_checkdepends(alpm_pkg_t *pkg);
+
+/** Returns a list of package make dependencies
+ * @param pkg a pointer to package
+ * @return a reference to an internal list of alpm_depend_t structures.
+ */
+alpm_list_t *alpm_pkg_get_makedepends(alpm_pkg_t *pkg);
+
 /** Returns the list of packages conflicting with pkg.
  * @param pkg a pointer to package
  * @return a reference to an internal list of alpm_depend_t structures.
@@ -910,12 +1249,6 @@ alpm_list_t *alpm_pkg_get_conflicts(alpm_pkg_t *pkg);
  */
 alpm_list_t *alpm_pkg_get_provides(alpm_pkg_t *pkg);
 
-/** Returns the list of available deltas for pkg.
- * @param pkg a pointer to package
- * @return a reference to an internal list of strings.
- */
-alpm_list_t *alpm_pkg_get_deltas(alpm_pkg_t *pkg);
-
 /** Returns the list of packages to be replaced by pkg.
  * @param pkg a pointer to package
  * @return a reference to an internal list of alpm_depend_t structures.
@@ -932,9 +1265,6 @@ alpm_list_t *alpm_pkg_get_replaces(alpm_pkg_t *pkg);
 alpm_filelist_t *alpm_pkg_get_files(alpm_pkg_t *pkg);
 
 /** Returns the list of files backed up when installing pkg.
- * The elements of the returned list have the form
- * "<filename>\t<md5sum>", where the given md5sum is that of
- * the file as provided by the package.
  * @param pkg a pointer to package
  * @return a reference to a list of alpm_backup_t objects
  */
@@ -958,7 +1288,7 @@ const char *alpm_pkg_get_base64_sig(alpm_pkg_t *pkg);
  * @param pkg a pointer to package
  * @return an enum member giving the validation method
  */
-alpm_pkgvalidation_t alpm_pkg_get_validation(alpm_pkg_t *pkg);
+int alpm_pkg_get_validation(alpm_pkg_t *pkg);
 
 /* End of alpm_pkg_t accessors */
 /* @} */
@@ -1016,8 +1346,6 @@ int alpm_pkg_has_scriptlet(alpm_pkg_t *pkg);
  */
 off_t alpm_pkg_download_size(alpm_pkg_t *newpkg);
 
-alpm_list_t *alpm_pkg_unused_deltas(alpm_pkg_t *pkg);
-
 /** Set install reason for a package in the local database.
  * The provided package object must be from the local database or this method
  * will fail. The write to the local database is performed immediately.
@@ -1043,7 +1371,7 @@ int alpm_pkg_set_reason(alpm_pkg_t *pkg, alpm_pkgreason_t reason);
  * @param path the path to search for in the package
  * @return a pointer to the matching file or NULL if not found
  */
-char *alpm_filelist_contains(alpm_filelist_t *filelist, const char *path);
+alpm_file_t *alpm_filelist_contains(alpm_filelist_t *filelist, const char *path);
 
 /*
  * Signatures
@@ -1055,6 +1383,12 @@ int alpm_db_check_pgp_signature(alpm_db_t *db, alpm_siglist_t *siglist);
 
 int alpm_siglist_cleanup(alpm_siglist_t *siglist);
 
+int alpm_decode_signature(const char *base64_data,
+		unsigned char **data, size_t *data_len);
+
+int alpm_extract_keyid(alpm_handle_t *handle, const char *identifier,
+		const unsigned char *sig, const size_t len, alpm_list_t **keys);
+
 /*
  * Groups
  */
@@ -1065,7 +1399,7 @@ alpm_list_t *alpm_find_group_pkgs(alpm_list_t *dbs, const char *name);
  * Sync
  */
 
-alpm_pkg_t *alpm_sync_newversion(alpm_pkg_t *pkg, alpm_list_t *dbs_sync);
+alpm_pkg_t *alpm_sync_get_new_version(alpm_pkg_t *pkg, alpm_list_t *dbs_sync);
 
 /** @addtogroup alpm_api_trans Transaction Functions
  * Functions to manipulate libalpm transactions
@@ -1076,8 +1410,7 @@ alpm_pkg_t *alpm_sync_newversion(alpm_pkg_t *pkg, alpm_list_t *dbs_sync);
 typedef enum _alpm_transflag_t {
 	/** Ignore dependency checks. */
 	ALPM_TRANS_FLAG_NODEPS = 1,
-	/** Ignore file conflicts and overwrite files. */
-	ALPM_TRANS_FLAG_FORCE = (1 << 1),
+	/* (1 << 1) flag can go here */
 	/** Delete files even if they are tagged as backup. */
 	ALPM_TRANS_FLAG_NOSAVE = (1 << 2),
 	/** Ignore version numbers when checking dependencies. */
@@ -1114,7 +1447,7 @@ typedef enum _alpm_transflag_t {
  * @param handle the context handle
  * @return the bitfield of transaction flags
  */
-alpm_transflag_t alpm_trans_get_flags(alpm_handle_t *handle);
+int alpm_trans_get_flags(alpm_handle_t *handle);
 
 /** Returns a list of packages added by the transaction.
  * @param handle the context handle
@@ -1130,10 +1463,10 @@ alpm_list_t *alpm_trans_get_remove(alpm_handle_t *handle);
 
 /** Initialize the transaction.
  * @param handle the context handle
- * @param flags flags of the transaction (like nodeps, etc)
+ * @param flags flags of the transaction (like nodeps, etc; see alpm_transflag_t)
  * @return 0 on success, -1 on error (pm_errno is set accordingly)
  */
-int alpm_trans_init(alpm_handle_t *handle, alpm_transflag_t flags);
+int alpm_trans_init(alpm_handle_t *handle, int flags);
 
 /** Prepare a transaction.
  * @param handle the context handle
@@ -1212,6 +1545,17 @@ alpm_list_t *alpm_checkconflicts(alpm_handle_t *handle, alpm_list_t *pkglist);
  */
 char *alpm_dep_compute_string(const alpm_depend_t *dep);
 
+/** Return a newly allocated dependency information parsed from a string
+ * @param depstring a formatted string, e.g. "glibc=2.12"
+ * @return a dependency info structure
+ */
+alpm_depend_t *alpm_dep_from_string(const char *depstring);
+
+/** Free a dependency info structure
+ * @param dep struct to free
+ */
+void alpm_dep_free(alpm_depend_t *dep);
+
 /** @} */
 
 /** @} */
@@ -1224,87 +1568,10 @@ char *alpm_dep_compute_string(const alpm_depend_t *dep);
 char *alpm_compute_md5sum(const char *filename);
 char *alpm_compute_sha256sum(const char *filename);
 
-/** @addtogroup alpm_api_errors Error Codes
- * @{
- */
-typedef enum _alpm_errno_t {
-	ALPM_ERR_MEMORY = 1,
-	ALPM_ERR_SYSTEM,
-	ALPM_ERR_BADPERMS,
-	ALPM_ERR_NOT_A_FILE,
-	ALPM_ERR_NOT_A_DIR,
-	ALPM_ERR_WRONG_ARGS,
-	ALPM_ERR_DISK_SPACE,
-	/* Interface */
-	ALPM_ERR_HANDLE_NULL,
-	ALPM_ERR_HANDLE_NOT_NULL,
-	ALPM_ERR_HANDLE_LOCK,
-	/* Databases */
-	ALPM_ERR_DB_OPEN,
-	ALPM_ERR_DB_CREATE,
-	ALPM_ERR_DB_NULL,
-	ALPM_ERR_DB_NOT_NULL,
-	ALPM_ERR_DB_NOT_FOUND,
-	ALPM_ERR_DB_INVALID,
-	ALPM_ERR_DB_INVALID_SIG,
-	ALPM_ERR_DB_VERSION,
-	ALPM_ERR_DB_WRITE,
-	ALPM_ERR_DB_REMOVE,
-	/* Servers */
-	ALPM_ERR_SERVER_BAD_URL,
-	ALPM_ERR_SERVER_NONE,
-	/* Transactions */
-	ALPM_ERR_TRANS_NOT_NULL,
-	ALPM_ERR_TRANS_NULL,
-	ALPM_ERR_TRANS_DUP_TARGET,
-	ALPM_ERR_TRANS_NOT_INITIALIZED,
-	ALPM_ERR_TRANS_NOT_PREPARED,
-	ALPM_ERR_TRANS_ABORT,
-	ALPM_ERR_TRANS_TYPE,
-	ALPM_ERR_TRANS_NOT_LOCKED,
-	/* Packages */
-	ALPM_ERR_PKG_NOT_FOUND,
-	ALPM_ERR_PKG_IGNORED,
-	ALPM_ERR_PKG_INVALID,
-	ALPM_ERR_PKG_INVALID_CHECKSUM,
-	ALPM_ERR_PKG_INVALID_SIG,
-	ALPM_ERR_PKG_OPEN,
-	ALPM_ERR_PKG_CANT_REMOVE,
-	ALPM_ERR_PKG_INVALID_NAME,
-	ALPM_ERR_PKG_INVALID_ARCH,
-	ALPM_ERR_PKG_REPO_NOT_FOUND,
-	/* Signatures */
-	ALPM_ERR_SIG_MISSING,
-	ALPM_ERR_SIG_INVALID,
-	/* Deltas */
-	ALPM_ERR_DLT_INVALID,
-	ALPM_ERR_DLT_PATCHFAILED,
-	/* Dependencies */
-	ALPM_ERR_UNSATISFIED_DEPS,
-	ALPM_ERR_CONFLICTING_DEPS,
-	ALPM_ERR_FILE_CONFLICTS,
-	/* Misc */
-	ALPM_ERR_RETRIEVE,
-	ALPM_ERR_INVALID_REGEX,
-	/* External library errors */
-	ALPM_ERR_LIBARCHIVE,
-	ALPM_ERR_LIBCURL,
-	ALPM_ERR_EXTERNAL_DOWNLOAD,
-	ALPM_ERR_GPGME
-} alpm_errno_t;
-
-/** Returns the current error code from the handle. */
-alpm_errno_t alpm_errno(alpm_handle_t *handle);
-
-/** Returns the string corresponding to an error number. */
-const char *alpm_strerror(alpm_errno_t err);
-
-/* End of alpm_api_errors */
-/** @} */
-
 alpm_handle_t *alpm_initialize(const char *root, const char *dbpath,
 		alpm_errno_t *err);
 int alpm_release(alpm_handle_t *handle);
+int alpm_unlock(alpm_handle_t *handle);
 
 enum alpm_caps {
 	ALPM_CAPABILITY_NLS = (1 << 0),
@@ -1313,7 +1580,12 @@ enum alpm_caps {
 };
 
 const char *alpm_version(void);
-enum alpm_caps alpm_capabilities(void);
+/* Return a bitfield of capabilities using values from 'enum alpm_caps' */
+int alpm_capabilities(void);
+
+void alpm_fileconflict_free(alpm_fileconflict_t *conflict);
+void alpm_depmissing_free(alpm_depmissing_t *miss);
+void alpm_conflict_free(alpm_conflict_t *conflict);
 
 /* End of alpm_api */
 /** @} */
@@ -1321,6 +1593,4 @@ enum alpm_caps alpm_capabilities(void);
 #ifdef __cplusplus
 }
 #endif
-#endif /* _ALPM_H */
-
-/* vim: set ts=2 sw=2 noet: */
+#endif /* ALPM_H */
