@@ -1,7 +1,7 @@
 /*
  *  alpm_list.c
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -131,6 +131,26 @@ alpm_list_t SYMEXPORT *alpm_list_append(alpm_list_t **list, void *data)
 	return ptr;
 }
 
+/**
+ * @brief Duplicate and append a string to a list.
+ *
+ * @param list the list to append to
+ * @param data the string to duplicate and append
+ *
+ * @return the newly added item
+ */
+alpm_list_t SYMEXPORT *alpm_list_append_strdup(alpm_list_t **list, const char *data)
+{
+	alpm_list_t *ret;
+	char *dup;
+	if((dup = strdup(data)) && (ret = alpm_list_append(list, dup))) {
+		return ret;
+	} else {
+		free(dup);
+		return NULL;
+	}
+}
+
 /**
  * @brief Add items to a list in sorted order.
  *
@@ -441,7 +461,10 @@ alpm_list_t SYMEXPORT *alpm_list_remove_dupes(const alpm_list_t *list)
 	alpm_list_t *newlist = NULL;
 	while(lp) {
 		if(!alpm_list_find_ptr(newlist, lp->data)) {
-			newlist = alpm_list_add(newlist, lp->data);
+			if(alpm_list_append(&newlist, lp->data) == NULL) {
+				alpm_list_free(newlist);
+				return NULL;
+			}
 		}
 		lp = lp->next;
 	}
@@ -460,7 +483,10 @@ alpm_list_t SYMEXPORT *alpm_list_strdup(const alpm_list_t *list)
 	const alpm_list_t *lp = list;
 	alpm_list_t *newlist = NULL;
 	while(lp) {
-		newlist = alpm_list_add(newlist, strdup(lp->data));
+		if(alpm_list_append_strdup(&newlist, lp->data) == NULL) {
+			FREELIST(newlist);
+			return NULL;
+		}
 		lp = lp->next;
 	}
 	return newlist;
@@ -478,7 +504,10 @@ alpm_list_t SYMEXPORT *alpm_list_copy(const alpm_list_t *list)
 	const alpm_list_t *lp = list;
 	alpm_list_t *newlist = NULL;
 	while(lp) {
-		newlist = alpm_list_add(newlist, lp->data);
+		if(alpm_list_append(&newlist, lp->data) == NULL) {
+			alpm_list_free(newlist);
+			return NULL;
+		}
 		lp = lp->next;
 	}
 	return newlist;
@@ -503,8 +532,15 @@ alpm_list_t SYMEXPORT *alpm_list_copy_data(const alpm_list_t *list,
 		void *newdata = malloc(size);
 		if(newdata) {
 			memcpy(newdata, lp->data, size);
-			newlist = alpm_list_add(newlist, newdata);
+			if(alpm_list_append(&newlist, newdata) == NULL) {
+				free(newdata);
+				FREELIST(newlist);
+				return NULL;
+			}
 			lp = lp->next;
+		} else {
+			FREELIST(newlist);
+			return NULL;
 		}
 	}
 	return newlist;
@@ -532,7 +568,10 @@ alpm_list_t SYMEXPORT *alpm_list_reverse(alpm_list_t *list)
 	list->prev = NULL;
 
 	while(lp) {
-		newlist = alpm_list_add(newlist, lp->data);
+		if(alpm_list_append(&newlist, lp->data) == NULL) {
+			alpm_list_free(newlist);
+			return NULL;
+		}
 		lp = lp->prev;
 	}
 	list->prev = backup; /* restore tail pointer */
@@ -803,5 +842,3 @@ void SYMEXPORT *alpm_list_to_array(const alpm_list_t *list, size_t n,
 }
 
 /** @} */
-
-/* vim: set noet: */
