@@ -1,7 +1,7 @@
 /*
  *  be_sync.c : backend for sync databases
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -37,7 +37,6 @@
 #include "alpm_list.h"
 #include "package.h"
 #include "handle.h"
-#include "delta.h"
 #include "deps.h"
 #include "dload.h"
 #include "filelist.h"
@@ -71,7 +70,7 @@ static char *get_sync_dir(alpm_handle_t *handle)
 
 static int sync_db_validate(alpm_db_t *db)
 {
-	alpm_siglevel_t level;
+	int siglevel;
 	const char *dbpath;
 
 	if(db->status & DB_STATUS_VALID || db->status & DB_STATUS_MISSING) {
@@ -104,20 +103,20 @@ static int sync_db_validate(alpm_db_t *db)
 
 	/* this takes into account the default verification level if UNKNOWN
 	 * was assigned to this db */
-	level = alpm_db_get_siglevel(db);
+	siglevel = alpm_db_get_siglevel(db);
 
-	if(level & ALPM_SIG_DATABASE) {
+	if(siglevel & ALPM_SIG_DATABASE) {
 		int retry, ret;
 		do {
 			retry = 0;
 			alpm_siglist_t *siglist;
 			ret = _alpm_check_pgp_helper(db->handle, dbpath, NULL,
-					level & ALPM_SIG_DATABASE_OPTIONAL, level & ALPM_SIG_DATABASE_MARGINAL_OK,
-					level & ALPM_SIG_DATABASE_UNKNOWN_OK, &siglist);
+					siglevel & ALPM_SIG_DATABASE_OPTIONAL, siglevel & ALPM_SIG_DATABASE_MARGINAL_OK,
+					siglevel & ALPM_SIG_DATABASE_UNKNOWN_OK, &siglist);
 			if(ret) {
 				retry = _alpm_process_siglist(db->handle, db->treename, siglist,
-						level & ALPM_SIG_DATABASE_OPTIONAL, level & ALPM_SIG_DATABASE_MARGINAL_OK,
-						level & ALPM_SIG_DATABASE_UNKNOWN_OK);
+						siglevel & ALPM_SIG_DATABASE_OPTIONAL, siglevel & ALPM_SIG_DATABASE_MARGINAL_OK,
+						siglevel & ALPM_SIG_DATABASE_UNKNOWN_OK);
 			}
 			alpm_siglist_cleanup(siglist);
 			free(siglist);
@@ -181,12 +180,12 @@ int SYMEXPORT alpm_db_update(int force, alpm_db_t *db)
 	int ret = -1;
 	mode_t oldmask;
 	alpm_handle_t *handle;
-	alpm_siglevel_t level;
+	int siglevel;
 
 	/* Sanity checks */
 	ASSERT(db != NULL, return -1);
 	handle = db->handle;
-	handle->pm_errno = 0;
+	handle->pm_errno = ALPM_ERR_OK;
 	ASSERT(db != handle->db_local, RET_ERR(handle, ALPM_ERR_WRONG_ARGS, -1));
 	ASSERT(db->servers != NULL, RET_ERR(handle, ALPM_ERR_SERVER_NONE, -1));
 
@@ -207,7 +206,7 @@ int SYMEXPORT alpm_db_update(int force, alpm_db_t *db)
 	/* make sure we have a sane umask */
 	oldmask = umask(0022);
 
-	level = alpm_db_get_siglevel(db);
+	siglevel = alpm_db_get_siglevel(db);
 
 	/* attempt to grab a lock */
 	if(_alpm_handle_lock(handle)) {
@@ -247,7 +246,7 @@ int SYMEXPORT alpm_db_update(int force, alpm_db_t *db)
 		_alpm_dload_payload_reset(&payload);
 		updated = (updated || ret == 0);
 
-		if(ret != -1 && updated && (level & ALPM_SIG_DATABASE)) {
+		if(ret != -1 && updated && (siglevel & ALPM_SIG_DATABASE)) {
 			/* an existing sig file is no good at this point */
 			char *sigpath = _alpm_sigpath(handle, _alpm_db_path(db));
 			if(!sigpath) {
@@ -292,7 +291,7 @@ int SYMEXPORT alpm_db_update(int force, alpm_db_t *db)
 
 			payload.handle = handle;
 			payload.force = 1;
-			payload.errors_ok = (level & ALPM_SIG_DATABASE_OPTIONAL);
+			payload.errors_ok = (siglevel & ALPM_SIG_DATABASE_OPTIONAL);
 
 			/* set hard upper limit of 16KiB */
 			payload.max_size = 16 * 1024;
@@ -330,7 +329,7 @@ int SYMEXPORT alpm_db_update(int force, alpm_db_t *db)
 		_alpm_log(handle, ALPM_LOG_DEBUG, "failed to sync db: %s\n",
 				alpm_strerror(handle->pm_errno));
 	} else {
-		handle->pm_errno = 0;
+		handle->pm_errno = ALPM_ERR_OK;
 	}
 
 	_alpm_handle_unlock(handle);
@@ -343,7 +342,7 @@ int SYMEXPORT alpm_db_update(int force, alpm_db_t *db)
 static int sync_db_read(alpm_db_t *db, struct archive *archive,
 		struct archive_entry *entry, alpm_pkg_t **likely_pkg);
 
-static alpm_pkgvalidation_t _sync_get_validation(alpm_pkg_t *pkg)
+static int _sync_get_validation(alpm_pkg_t *pkg)
 {
 	if(pkg->validation) {
 		return pkg->validation;
@@ -413,7 +412,10 @@ static alpm_pkg_t *load_pkg_for_entry(alpm_db_t *db, const char *entryname,
 		/* add to the collection */
 		_alpm_log(db->handle, ALPM_LOG_FUNCTION, "adding '%s' to package cache for db '%s'\n",
 				pkg->name, db->treename);
-		db->pkgcache = _alpm_pkghash_add(db->pkgcache, pkg);
+		if(_alpm_pkghash_add(&db->pkgcache, pkg) == NULL) {
+			_alpm_pkg_free(pkg);
+			RET_ERR(db->handle, ALPM_ERR_MEMORY, NULL);
+		}
 	} else {
 		free(pkgname);
 		free(pkgver);
@@ -424,9 +426,8 @@ static alpm_pkg_t *load_pkg_for_entry(alpm_db_t *db, const char *entryname,
 
 /* This function doesn't work as well as one might think, as size of database
  * entries varies considerably. Adding signatures nearly doubles the size of a
- * single entry; deltas also can make for large variations in size. These
- * current values are heavily influenced by Arch Linux; databases with no
- * deltas and a single signature per package. */
+ * single entry. These  current values are heavily influenced by Arch Linux;
+ * databases with a single signature per package. */
 static size_t estimate_package_count(struct stat *st, struct archive *archive)
 {
 	int per_package;
@@ -465,6 +466,7 @@ static int sync_db_populate(alpm_db_t *db)
 	size_t est_count, count;
 	int fd;
 	int ret = 0;
+	int archive_ret;
 	struct stat buf;
 	struct archive *archive;
 	struct archive_entry *entry;
@@ -485,6 +487,8 @@ static int sync_db_populate(alpm_db_t *db)
 	fd = _alpm_open_archive(db->handle, dbpath, &buf,
 			&archive, ALPM_ERR_DB_OPEN);
 	if(fd < 0) {
+		db->status &= ~DB_STATUS_VALID;
+		db->status |= DB_STATUS_INVALID;
 		return -1;
 	}
 	est_count = estimate_package_count(&buf, archive);
@@ -502,20 +506,26 @@ static int sync_db_populate(alpm_db_t *db)
 		goto cleanup;
 	}
 
-	while(archive_read_next_header(archive, &entry) == ARCHIVE_OK) {
+	while((archive_ret = archive_read_next_header(archive, &entry)) == ARCHIVE_OK) {
 		mode_t mode = archive_entry_mode(entry);
-		if(S_ISDIR(mode)) {
-			continue;
-		} else {
-			/* we have desc, depends or deltas - parse it */
+		if(!S_ISDIR(mode)) {
+			/* we have desc or depends - parse it */
 			if(sync_db_read(db, archive, entry, &pkg) != 0) {
 				_alpm_log(db->handle, ALPM_LOG_ERROR,
 						_("could not parse package description file '%s' from db '%s'\n"),
 						archive_entry_pathname(entry), db->treename);
-				continue;
+				ret = -1;
 			}
 		}
 	}
+	if(archive_ret != ARCHIVE_EOF) {
+		_alpm_log(db->handle, ALPM_LOG_ERROR, _("could not read db '%s' (%s)\n"),
+				db->treename, archive_error_string(archive));
+		_alpm_db_free_pkgcache(db);
+		db->handle->pm_errno = ALPM_ERR_LIBARCHIVE;
+		ret = -1;
+		goto cleanup;
+	}
 
 	count = alpm_list_count(db->pkgcache->list);
 	if(count > 0) {
@@ -611,7 +621,7 @@ static int sync_db_read(alpm_db_t *db, struct archive *archive,
 
 	if(pkg == NULL) {
 		_alpm_log(db->handle, ALPM_LOG_DEBUG,
-				"entry %s could not be loaded into %s sync database",
+				"entry %s could not be loaded into %s sync database\n",
 				entryname, db->treename);
 		return -1;
 	}
@@ -620,13 +630,12 @@ static int sync_db_read(alpm_db_t *db, struct archive *archive,
 		/* A file exists outside of a subdirectory. This isn't a read error, so return
 		 * success and try to continue on. */
 		_alpm_log(db->handle, ALPM_LOG_WARNING, _("unknown database file: %s\n"),
-				filename);
+				entryname);
 		return 0;
 	}
 
 	if(strcmp(filename, "desc") == 0 || strcmp(filename, "depends") == 0
-			|| strcmp(filename, "files") == 0
-			|| (strcmp(filename, "deltas") == 0 && db->handle->deltaratio > 0.0) ) {
+			|| strcmp(filename, "files") == 0) {
 		int ret;
 		while((ret = _alpm_archive_fgets(archive, &buf)) == ARCHIVE_OK) {
 			char *line = buf.line;
@@ -688,29 +697,13 @@ static int sync_db_read(alpm_db_t *db, struct archive *archive,
 			} else if(strcmp(line, "%OPTDEPENDS%") == 0) {
 				READ_AND_SPLITDEP(pkg->optdepends);
 			} else if(strcmp(line, "%MAKEDEPENDS%") == 0) {
-				/* currently unused */
-				while(1) {
-					READ_NEXT();
-					if(strlen(line) == 0) break;
-				}
+				READ_AND_SPLITDEP(pkg->makedepends);
 			} else if(strcmp(line, "%CHECKDEPENDS%") == 0) {
-				/* currently unused */
-				while(1) {
-					READ_NEXT();
-					if(strlen(line) == 0) break;
-				}
+				READ_AND_SPLITDEP(pkg->checkdepends);
 			} else if(strcmp(line, "%CONFLICTS%") == 0) {
 				READ_AND_SPLITDEP(pkg->conflicts);
 			} else if(strcmp(line, "%PROVIDES%") == 0) {
 				READ_AND_SPLITDEP(pkg->provides);
-			} else if(strcmp(line, "%DELTAS%") == 0) {
-				/* Different than the rest because of the _alpm_delta_parse call. */
-				while(1) {
-					READ_NEXT();
-					if(strlen(line) == 0) break;
-					pkg->deltas = alpm_list_add(pkg->deltas,
-							_alpm_delta_parse(db->handle, line));
-				}
 			} else if(strcmp(line, "%FILES%") == 0) {
 				/* TODO: this could lazy load if there is future demand */
 				size_t files_count = 0, files_size = 0;
@@ -735,21 +728,18 @@ static int sync_db_read(alpm_db_t *db, struct archive *archive,
 				/* attempt to hand back any memory we don't need */
 				if(files_count > 0) {
 					files = realloc(files, sizeof(alpm_file_t) * files_count);
-					/* make sure the list is sorted */
-					qsort(files, files_count, sizeof(alpm_file_t), _alpm_files_cmp);
 				} else {
 					FREE(files);
 				}
 				pkg->files.count = files_count;
 				pkg->files.files = files;
+				_alpm_filelist_sort(&pkg->files);
 			}
 		}
 		if(ret != ARCHIVE_EOF) {
 			goto error;
 		}
 		*likely_pkg = pkg;
-	} else if(strcmp(filename, "deltas") == 0) {
-		/* skip reading delta files if UseDelta is unset */
 	} else {
 		/* unknown database file */
 		_alpm_log(db->handle, ALPM_LOG_DEBUG, "unknown database file: %s\n", filename);
@@ -769,15 +759,15 @@ struct db_operations sync_db_ops = {
 };
 
 alpm_db_t *_alpm_db_register_sync(alpm_handle_t *handle, const char *treename,
-		alpm_siglevel_t level)
+		int level)
 {
 	alpm_db_t *db;
 
 	_alpm_log(handle, ALPM_LOG_DEBUG, "registering sync database '%s'\n", treename);
 
 #ifndef HAVE_LIBGPGME
-	if(level != ALPM_SIG_USE_DEFAULT) {
-		RET_ERR(handle, ALPM_ERR_WRONG_ARGS, NULL);
+	if(level != 0 && level != ALPM_SIG_USE_DEFAULT) {
+		RET_ERR(handle, ALPM_ERR_MISSING_CAPABILITY_SIGNATURES, NULL);
 	}
 #endif
 
@@ -794,5 +784,3 @@ alpm_db_t *_alpm_db_register_sync(alpm_handle_t *handle, const char *treename,
 	handle->dbs_sync = alpm_list_add(handle->dbs_sync, db);
 	return db;
 }
-
-/* vim: set noet: */
