@@ -53,9 +53,11 @@ static const char *get_filename(const char *url)
 {
 	char *filename = strrchr(url, '/');
 	if(filename != NULL) {
-		filename++;
+		return filename + 1;
 	}
-	return filename;
+
+	/* no slash found, it's a filename */
+	return url;
 }
 
 static char *get_fullpath(const char *path, const char *filename,
@@ -259,6 +261,7 @@ static void curl_set_handle_opts(struct dload_payload *payload,
 	curl_easy_setopt(curl, CURLOPT_URL, payload->fileurl);
 	curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, error_buffer);
 	curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 10L);
+	curl_easy_setopt(curl, CURLOPT_MAXREDIRS, 10L);
 	curl_easy_setopt(curl, CURLOPT_FILETIME, 1L);
 	curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0L);
 	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
@@ -453,6 +456,7 @@ static int curl_download_internal(struct dload_payload *payload,
 			if(payload->respcode >= 400) {
 				payload->unlink_on_fail = 1;
 				if(!payload->errors_ok) {
+					handle->pm_errno = ALPM_ERR_RETRIEVE;
 					/* non-translated message is same as libcurl */
 					snprintf(error_buffer, sizeof(error_buffer),
 							"The requested URL returned error: %ld", payload->respcode);
@@ -534,7 +538,8 @@ static int curl_download_internal(struct dload_payload *payload,
 		if(payload->content_disp_name) {
 			/* content-disposition header has a better name for our file */
 			free(payload->destfile_name);
-			payload->destfile_name = get_fullpath(localpath, payload->content_disp_name, "");
+			payload->destfile_name = get_fullpath(localpath,
+				get_filename(payload->content_disp_name), "");
 		} else {
 			const char *effective_filename = strrchr(effective_url, '/');
 			if(effective_filename && strlen(effective_filename) > 2) {
