@@ -1,7 +1,7 @@
 /*
  *  error.c
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -130,11 +130,6 @@ const char SYMEXPORT *alpm_strerror(alpm_errno_t err)
 			return _("missing PGP signature");
 		case ALPM_ERR_SIG_INVALID:
 			return _("invalid PGP signature");
-		/* Deltas */
-		case ALPM_ERR_DLT_INVALID:
-			return _("invalid or corrupted delta");
-		case ALPM_ERR_DLT_PATCHFAILED:
-			return _("delta patch failed");
 		/* Dependencies */
 		case ALPM_ERR_UNSATISFIED_DEPS:
 			return _("could not satisfy dependencies");
@@ -159,10 +154,11 @@ const char SYMEXPORT *alpm_strerror(alpm_errno_t err)
 			return _("gpgme error");
 		case ALPM_ERR_EXTERNAL_DOWNLOAD:
 			return _("error invoking external downloader");
+		/* Missing compile-time features */
+		case ALPM_ERR_MISSING_CAPABILITY_SIGNATURES:
+				return _("compiled without signature support");
 		/* Unknown error! */
 		default:
 			return _("unexpected error");
 	}
 }
-
-/* vim: set noet: */
