@@ -1,7 +1,7 @@
 /*
  *  filelist.h
  *
- *  Copyright (c) 2012-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2012-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,8 +27,6 @@ alpm_list_t *_alpm_filelist_difference(alpm_filelist_t *filesA,
 alpm_list_t *_alpm_filelist_intersection(alpm_filelist_t *filesA,
 		alpm_filelist_t *filesB);
 
-int _alpm_files_cmp(const void *f1, const void *f2);
+void _alpm_filelist_sort(alpm_filelist_t *filelist);
 
 #endif /* ALPM_FILELIST_H */
-
-/* vim: set noet: */
