@@ -34,7 +34,6 @@
 #include "alpm_list.h"
 #include "util.h"
 #include "log.h"
-#include "delta.h"
 #include "trans.h"
 #include "alpm.h"
 #include "deps.h"
@@ -44,7 +43,6 @@ alpm_handle_t *_alpm_handle_new(void)
 	alpm_handle_t *handle;
 
 	CALLOC(handle, 1, sizeof(alpm_handle_t), return NULL);
-	handle->deltaratio = 0.0;
 	handle->lockfd = -1;
 
 	return handle;
@@ -75,8 +73,6 @@ void _alpm_handle_free(alpm_handle_t *handle)
 	FREELIST(handle->known_keys);
 #endif
 
-	regfree(&handle->delta_regex);
-
 	/* free memory */
 	_alpm_trans_free(handle->trans);
 	FREE(handle->root);
@@ -303,12 +299,6 @@ const char SYMEXPORT *alpm_option_get_arch(alpm_handle_t *handle)
 	return handle->arch;
 }
 
-double SYMEXPORT alpm_option_get_deltaratio(alpm_handle_t *handle)
-{
-	CHECK_HANDLE(handle, return -1);
-	return handle->deltaratio;
-}
-
 int SYMEXPORT alpm_option_get_checkspace(alpm_handle_t *handle)
 {
 	CHECK_HANDLE(handle, return -1);
@@ -755,16 +745,6 @@ int SYMEXPORT alpm_option_set_arch(alpm_handle_t *handle, const char *arch)
 	return 0;
 }
 
-int SYMEXPORT alpm_option_set_deltaratio(alpm_handle_t *handle, double ratio)
-{
-	CHECK_HANDLE(handle, return -1);
-	if(ratio < 0.0 || ratio > 2.0) {
-		RET_ERR(handle, ALPM_ERR_WRONG_ARGS, -1);
-	}
-	handle->deltaratio = ratio;
-	return 0;
-}
-
 alpm_db_t SYMEXPORT *alpm_get_localdb(alpm_handle_t *handle)
 {
 	CHECK_HANDLE(handle, return NULL);
@@ -803,11 +783,14 @@ int SYMEXPORT alpm_option_set_default_siglevel(alpm_handle_t *handle,
 		int level)
 {
 	CHECK_HANDLE(handle, return -1);
+	if(level == ALPM_SIG_USE_DEFAULT) {
+		RET_ERR(handle, ALPM_ERR_WRONG_ARGS, -1);
+	}
 #ifdef HAVE_LIBGPGME
 	handle->siglevel = level;
 #else
-	if(level != 0 && level != ALPM_SIG_USE_DEFAULT) {
-		RET_ERR(handle, ALPM_ERR_WRONG_ARGS, -1);
+	if(level != 0) {
+		RET_ERR(handle, ALPM_ERR_MISSING_CAPABILITY_SIGNATURES, -1);
 	}
 #endif
 	return 0;
@@ -827,7 +810,7 @@ int SYMEXPORT alpm_option_set_local_file_siglevel(alpm_handle_t *handle,
 	handle->localfilesiglevel = level;
 #else
 	if(level != 0 && level != ALPM_SIG_USE_DEFAULT) {
-		RET_ERR(handle, ALPM_ERR_WRONG_ARGS, -1);
+		RET_ERR(handle, ALPM_ERR_MISSING_CAPABILITY_SIGNATURES, -1);
 	}
 #endif
 	return 0;
@@ -851,7 +834,7 @@ int SYMEXPORT alpm_option_set_remote_file_siglevel(alpm_handle_t *handle,
 	handle->remotefilesiglevel = level;
 #else
 	if(level != 0 && level != ALPM_SIG_USE_DEFAULT) {
-		RET_ERR(handle, ALPM_ERR_WRONG_ARGS, -1);
+		RET_ERR(handle, ALPM_ERR_MISSING_CAPABILITY_SIGNATURES, -1);
 	}
 #endif
 	return 0;
@@ -873,6 +856,8 @@ int SYMEXPORT alpm_option_set_disable_dl_timeout(alpm_handle_t *handle,
 	CHECK_HANDLE(handle, return -1);
 #ifdef HAVE_LIBCURL
 	handle->disable_dl_timeout = disable_dl_timeout;
+#else
+	(void)disable_dl_timeout; /* silence unused variable warnings */
 #endif
 	return 0;
 }
