@@ -1,7 +1,7 @@
 /*
  *  log.c
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #include <stdarg.h>
 #include <errno.h>
 #include <syslog.h>
+#include <time.h>
 
 /* libalpm */
 #include "log.h"
@@ -38,11 +39,12 @@ static int _alpm_log_leader(FILE *f, const char *prefix)
 {
 	time_t t = time(NULL);
 	struct tm *tm = localtime(&t);
+	int length = 32;
+	char timestamp[length];
 
 	/* Use ISO-8601 date format */
-	return fprintf(f, "[%04d-%02d-%02d %02d:%02d] [%s] ",
-			tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-			tm->tm_hour, tm->tm_min, prefix);
+	strftime(timestamp,length,"%FT%T%z", tm);
+	return fprintf(f, "[%s] [%s] ", timestamp, prefix);
 }
 
 /** A printf-like function for logging.
@@ -121,5 +123,3 @@ void _alpm_log(alpm_handle_t *handle, alpm_loglevel_t flag, const char *fmt, ...
 	handle->logcb(flag, fmt, args);
 	va_end(args);
 }
-
-/* vim: set noet: */
