@@ -1,7 +1,7 @@
 /*
  *  remove.c
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
@@ -74,7 +74,8 @@ int SYMEXPORT alpm_remove_pkg(alpm_handle_t *handle, alpm_pkg_t *pkg)
 	pkgname = pkg->name;
 
 	if(alpm_pkg_find(trans->remove, pkgname)) {
-		RET_ERR(handle, ALPM_ERR_TRANS_DUP_TARGET, -1);
+		_alpm_log(handle, ALPM_LOG_DEBUG, "skipping duplicate target: %s\n", pkgname);
+		return 0;
 	}
 
 	_alpm_log(handle, ALPM_LOG_DEBUG, "adding package %s to the transaction remove list\n",
@@ -404,14 +405,22 @@ static void shift_pacsave(alpm_handle_t *handle, const char *file)
 	/* Shift pacsaves */
 	unsigned long i;
 	for(i = log_max + 1; i > 1; i--) {
-		snprintf(oldfile, PATH_MAX, "%s.pacsave.%lu", file, i-1);
-		snprintf(newfile, PATH_MAX, "%s.pacsave.%lu", file, i);
+		if(snprintf(oldfile, PATH_MAX, "%s.pacsave.%lu", file, i-1) >= PATH_MAX
+				|| snprintf(newfile, PATH_MAX, "%s.pacsave.%lu", file, i) >= PATH_MAX) {
+			_alpm_log(handle, ALPM_LOG_ERROR,
+					_("could not backup %s due to PATH_MAX overflow\n"), file);
+			goto cleanup;
+		}
 		rename(oldfile, newfile);
 	}
 
-	snprintf(oldfile, PATH_MAX, "%s.pacsave", file);
+	if(snprintf(oldfile, PATH_MAX, "%s.pacsave", file) >= PATH_MAX
+			|| snprintf(newfile, PATH_MAX, "%s.1", oldfile) >= PATH_MAX) {
+		_alpm_log(handle, ALPM_LOG_ERROR,
+				_("could not backup %s due to PATH_MAX overflow\n"), file);
+		goto cleanup;
+	}
 	if(stat(oldfile, &st) == 0) {
-		snprintf(newfile, PATH_MAX, "%s.1", oldfile);
 		rename(oldfile, newfile);
 	}
 
@@ -779,5 +788,3 @@ int _alpm_remove_packages(alpm_handle_t *handle, int run_ldconfig)
 
 	return ret;
 }
-
-/* vim: set noet: */
