@@ -1,7 +1,7 @@
 /*
  *  signing.c
  *
- *  Copyright (c) 2008-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2008-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -182,7 +182,7 @@ static int init_gpgme(alpm_handle_t *handle)
 #endif
 	/* NOTE:
 	 * The GPGME library installs a SIGPIPE signal handler automatically if
-	 * the default signal hander is in use. The only time we set a handler
+	 * the default signal handler is in use. The only time we set a handler
 	 * for SIGPIPE is in dload.c, and we reset it when we are done. Given that
 	 * we do this, we can let GPGME do its automagic. However, if we install
 	 * a library-wide SIGPIPE handler, we will have to be careful.
@@ -258,7 +258,46 @@ error:
 }
 
 /**
- * Search for a GPG key in a remote location.
+ * Import a key from a Web Key Directory (WKD) into the local keyring using.
+ * This requires GPGME to call the gpg binary.
+ * @param handle the context handle
+ * @param email the email address of the key to import
+ * @return 0 on success, -1 on error
+ */
+static int key_import_wkd(alpm_handle_t *handle, const char *email)
+{
+	gpgme_error_t gpg_err;
+	gpgme_ctx_t ctx;
+	gpgme_keylist_mode_t mode;
+	gpgme_key_t key;
+	int ret = -1;
+
+	memset(&ctx, 0, sizeof(ctx));
+	gpg_err = gpgme_new(&ctx);
+	CHECK_ERR();
+
+	mode = gpgme_get_keylist_mode(ctx);
+	mode |= GPGME_KEYLIST_MODE_LOCATE;
+	gpg_err = gpgme_set_keylist_mode(ctx, mode);
+	CHECK_ERR();
+
+	_alpm_log(handle, ALPM_LOG_DEBUG, _("looking up key %s using WKD\n"), email);
+	gpg_err = gpgme_get_key(ctx, email, &key, 0);
+	if(gpg_err_code(gpg_err) == GPG_ERR_NO_ERROR) {
+		ret = 0;
+	}
+	gpgme_key_unref(key);
+
+gpg_error:
+	if(ret != 0) {
+		_alpm_log(handle, ALPM_LOG_DEBUG, _("gpg error: %s\n"), gpgme_strerror(gpg_err));
+	}
+	gpgme_release(ctx);
+	return ret;
+}
+
+/**
+ * Search for a GPG key on a keyserver.
  * This requires GPGME to call the gpg binary and have a keyserver previously
  * defined in a gpg.conf configuration file.
  * @param handle the context handle
@@ -266,7 +305,7 @@ error:
  * @param pgpkey storage location for the given key if found
  * @return 1 on success, 0 on key not found, -1 on error
  */
-static int key_search(alpm_handle_t *handle, const char *fpr,
+static int key_search_keyserver(alpm_handle_t *handle, const char *fpr,
 		alpm_pgpkey_t *pgpkey)
 {
 	gpgme_error_t gpg_err;
@@ -386,10 +425,10 @@ gpg_error:
 /**
  * Import a key into the local keyring.
  * @param handle the context handle
- * @param key the key to import, likely retrieved from #key_search
+ * @param key the key to import, likely retrieved from #key_search_keyserver
  * @return 0 on success, -1 on error
  */
-static int key_import(alpm_handle_t *handle, alpm_pgpkey_t *key)
+static int key_import_keyserver(alpm_handle_t *handle, alpm_pgpkey_t *key)
 {
 	gpgme_error_t gpg_err;
 	gpgme_ctx_t ctx;
@@ -430,45 +469,85 @@ gpg_error:
 	return ret;
 }
 
+/** Extract the email address from a user ID
+ * @param uid the user ID to parse in the form "Example Name <email@address.invalid>"
+ * @param email to hold email address
+ * @return 0 on success, -1 on error
+ */
+static int email_from_uid(const char *uid, char **email)
+{
+       char *start, *end;
+
+       if (uid == NULL) {
+               email = NULL;
+               return -1;
+       }
+
+       start = strrchr(uid, '<');
+       if(start) {
+               end = strrchr(start, '>');
+       }
+
+       if(start && end) {
+               STRNDUP(*email, start+1, end-start-1, return -1);
+               return 0;
+       } else {
+               email = NULL;
+               return -1;
+       }
+}
+
 /**
  * Import a key defined by a fingerprint into the local keyring.
  * @param handle the context handle
+ * @param uid a user ID of the key to import
  * @param fpr the fingerprint key ID to import
  * @return 0 on success, -1 on error
  */
-int _alpm_key_import(alpm_handle_t *handle, const char *fpr)
+int _alpm_key_import(alpm_handle_t *handle, const char *uid, const char *fpr)
 {
 	int ret = -1;
 	alpm_pgpkey_t fetch_key;
-	memset(&fetch_key, 0, sizeof(fetch_key));
+	char *email;
 
-	if(key_search(handle, fpr, &fetch_key) == 1) {
-		_alpm_log(handle, ALPM_LOG_DEBUG,
-				"unknown key, found %s on keyserver\n", fetch_key.uid);
-		if(!_alpm_access(handle, handle->gpgdir, "pubring.gpg", W_OK)) {
-			alpm_question_import_key_t question = {
+	if(_alpm_access(handle, handle->gpgdir, "pubring.gpg", W_OK)) {
+		/* no chance of import succeeding if pubring isn't writable */
+		_alpm_log(handle, ALPM_LOG_ERROR, _("keyring is not writable\n"));
+		return -1;
+	}
+
+	memset(&fetch_key, 0, sizeof(fetch_key));
+	STRDUP(fetch_key.uid, uid, return -1);
+	STRDUP(fetch_key.fingerprint, fpr, return -1);
+
+	alpm_question_import_key_t question = {
 				.type = ALPM_QUESTION_IMPORT_KEY,
 				.import = 0,
 				.key = &fetch_key
 			};
-			QUESTION(handle, &question);
-			if(question.import) {
-				if(key_import(handle, &fetch_key) == 0) {
+	QUESTION(handle, &question);
+	if(question.import) {
+		/* Try to import the key from a WKD first */
+		if(email_from_uid(uid, &email) == 0) {
+			ret = key_import_wkd(handle, email);
+		}
+
+		/* If importing from the WKD fails, fall back to keyserver lookup */
+		if(ret != 0) {
+			if(key_search_keyserver(handle, fpr, &fetch_key) == 1) {
+				_alpm_log(handle, ALPM_LOG_DEBUG,
+						_("key \"%s\" on keyserver\n"), fetch_key.uid);
+				if(key_import_keyserver(handle, &fetch_key) == 0) {
 					ret = 0;
 				} else {
 					_alpm_log(handle, ALPM_LOG_ERROR,
 							_("key \"%s\" could not be imported\n"), fetch_key.uid);
 				}
+			} else {
+				_alpm_log(handle, ALPM_LOG_ERROR,
+						_("key \"%s\" could not be looked up remotely\n"), fpr);
 			}
-		} else {
-			/* keyring directory was not writable, so we don't even try */
-			_alpm_log(handle, ALPM_LOG_WARNING,
-					_("key %s, \"%s\" found on keyserver, keyring is not writable\n"),
-					fetch_key.fingerprint, fetch_key.uid);
 		}
-	} else {
-		_alpm_log(handle, ALPM_LOG_ERROR,
-				_("key \"%s\" could not be looked up remotely\n"), fpr);
 	}
 	gpgme_key_unref(fetch_key.data);
 
@@ -709,19 +788,24 @@ error:
 }
 
 #else /* HAVE_LIBGPGME */
-int _alpm_key_in_keychain(alpm_handle_t UNUSED *handle, const char UNUSED *fpr)
+int _alpm_key_in_keychain(alpm_handle_t *handle, const char UNUSED *fpr)
 {
+	handle->pm_errno = ALPM_ERR_MISSING_CAPABILITY_SIGNATURES;
 	return -1;
 }
 
-int _alpm_key_import(alpm_handle_t UNUSED *handle, const char UNUSED *fpr)
+int _alpm_key_import(alpm_handle_t *handle, const char UNUSED *uid,
+		const char UNUSED *fpr)
 {
+	handle->pm_errno = ALPM_ERR_MISSING_CAPABILITY_SIGNATURES;
 	return -1;
 }
 
-int _alpm_gpgme_checksig(alpm_handle_t UNUSED *handle, const char UNUSED *path,
-		const char UNUSED *base64_sig, alpm_siglist_t UNUSED *siglist)
+int _alpm_gpgme_checksig(alpm_handle_t *handle, const char UNUSED *path,
+		const char UNUSED *base64_sig, alpm_siglist_t *siglist)
 {
+	siglist->count = 0;
+	handle->pm_errno = ALPM_ERR_MISSING_CAPABILITY_SIGNATURES;
 	return -1;
 }
 #endif /* HAVE_LIBGPGME */
@@ -775,7 +859,7 @@ int _alpm_check_pgp_helper(alpm_handle_t *handle, const char *path,
 	if(ret && handle->pm_errno == ALPM_ERR_SIG_MISSING) {
 		if(optional) {
 			_alpm_log(handle, ALPM_LOG_DEBUG, "missing optional signature\n");
-			handle->pm_errno = 0;
+			handle->pm_errno = ALPM_ERR_OK;
 			ret = 0;
 		} else {
 			_alpm_log(handle, ALPM_LOG_DEBUG, "missing required signature\n");
@@ -900,7 +984,7 @@ int _alpm_process_siglist(alpm_handle_t *handle, const char *identifier,
 				_alpm_log(handle, ALPM_LOG_ERROR,
 						_("%s: key \"%s\" is unknown\n"), identifier, name);
 
-				if(_alpm_key_import(handle, result->key.fingerprint) == 0) {
+				if(_alpm_key_import(handle, result->key.uid, result->key.fingerprint) == 0) {
 					retry = 1;
 				}
 
@@ -935,7 +1019,7 @@ int SYMEXPORT alpm_pkg_check_pgp_signature(alpm_pkg_t *pkg,
 {
 	ASSERT(pkg != NULL, return -1);
 	ASSERT(siglist != NULL, RET_ERR(pkg->handle, ALPM_ERR_WRONG_ARGS, -1));
-	pkg->handle->pm_errno = 0;
+	pkg->handle->pm_errno = ALPM_ERR_OK;
 
 	return _alpm_gpgme_checksig(pkg->handle, pkg->filename,
 			pkg->base64_sig, siglist);
@@ -952,7 +1036,7 @@ int SYMEXPORT alpm_db_check_pgp_signature(alpm_db_t *db,
 {
 	ASSERT(db != NULL, return -1);
 	ASSERT(siglist != NULL, RET_ERR(db->handle, ALPM_ERR_WRONG_ARGS, -1));
-	db->handle->pm_errno = 0;
+	db->handle->pm_errno = ALPM_ERR_OK;
 
 	return _alpm_gpgme_checksig(db->handle, _alpm_db_path(db), NULL, siglist);
 }
@@ -986,6 +1070,64 @@ int SYMEXPORT alpm_siglist_cleanup(alpm_siglist_t *siglist)
 	return 0;
 }
 
+/* Check to avoid out of boundary reads */
+static size_t length_check(size_t length, size_t position, size_t a,
+		alpm_handle_t *handle, const char *identifier)
+{
+	if( a == 0 || length - position <= a) {
+		_alpm_log(handle, ALPM_LOG_ERROR,
+				_("%s: signature format error\n"), identifier);
+		return -1;
+	} else {
+		return 0;
+	}
+}
+
+static int parse_subpacket(alpm_handle_t *handle, const char *identifier,
+		const unsigned char *sig, const size_t len, const size_t pos,
+		const size_t plen, alpm_list_t **keys)
+{
+		size_t slen;
+		size_t spos = pos;
+
+		while(spos < pos + plen) {
+			if(sig[spos] < 192) {
+				slen = sig[spos];
+				spos = spos + 1;
+			} else if(sig[spos] < 255) {
+				if(length_check(len, spos, 2, handle, identifier) != 0){
+					return -1;
+				}
+				slen = (sig[spos] << 8) | sig[spos + 1];
+				spos = spos + 2;
+			} else {
+				if(length_check(len, spos, 5, handle, identifier) != 0) {
+					return -1;
+				}
+				slen = (sig[spos + 1] << 24) | (sig[spos + 2] << 16) | (sig[spos + 3] << 8) | sig[spos + 4];
+				spos = spos + 5;
+			}
+			if(sig[spos] == 16) {
+				/* issuer key ID */
+				char key[17];
+				size_t i;
+				if(length_check(len, spos, 8, handle, identifier) != 0) {
+					return -1;
+				}
+				for (i = 0; i < 8; i++) {
+					sprintf(&key[i * 2], "%02X", sig[spos + i + 1]);
+				}
+				*keys = alpm_list_add(*keys, strdup(key));
+				break;
+			}
+			if(length_check(len, spos, slen, handle, identifier) != 0) {
+				return -1;
+			}
+			spos = spos + slen;
+		}
+		return 0;
+}
+
 /**
  * Extract the Issuer Key ID from a signature
  * @param sig PGP signature
@@ -996,107 +1138,123 @@ int SYMEXPORT alpm_siglist_cleanup(alpm_siglist_t *siglist)
 int SYMEXPORT alpm_extract_keyid(alpm_handle_t *handle, const char *identifier,
 		const unsigned char *sig, const size_t len, alpm_list_t **keys)
 {
-	size_t pos, spos, blen, hlen, ulen, slen;
+	size_t pos, blen, hlen, ulen;
 	pos = 0;
 
 	while(pos < len) {
 		if(!(sig[pos] & 0x80)) {
 			_alpm_log(handle, ALPM_LOG_ERROR,
-					_("%s: signature format error"), identifier);
+					_("%s: signature format error\n"), identifier);
 			return -1;
 		}
 
 		if(sig[pos] & 0x40) {
-			/* "new" packet format is not supported */
-			_alpm_log(handle, ALPM_LOG_ERROR,
-					_("%s: unsupported signature format"), identifier);
-			return -1;
-		}
+			/* new packet format */
+			if(length_check(len, pos, 1, handle, identifier) != 0) {
+				return -1;
+			}
+			pos = pos + 1;
 
-		if(((sig[pos] & 0x3f) >> 2) != 2) {
-			/* signature is not a "Signature Packet" */
-			_alpm_log(handle, ALPM_LOG_ERROR,
-					_("%s: signature format error"), identifier);
-			return -1;
-		}
-
-		switch(sig[pos] & 0x03) {
-			case 0:
-				blen = sig[pos + 1];
+			if(sig[pos] < 192) {
+				if(length_check(len, pos, 1, handle, identifier) != 0) {
+					return -1;
+				}
+				blen = sig[pos];
+				pos = pos + 1;
+			} else if (sig[pos] < 224) {
+				if(length_check(len, pos, 2, handle, identifier) != 0) {
+					return -1;
+				}
+				blen = ((sig[pos] - 192) << 8) + sig[pos + 1] + 192;
 				pos = pos + 2;
-				break;
-
-			case 1:
-				blen = (sig[pos + 1] << 8) | sig[pos + 2];
-				pos = pos + 3;
-				break;
-
-			case 2:
+			} else if (sig[pos] == 255) {
+				if(length_check(len, pos, 5, handle, identifier)) {
+					return -1;
+				}
 				blen = (sig[pos + 1] << 24) | (sig[pos + 2] << 16) | (sig[pos + 3] << 8) | sig[pos + 4];
 				pos = pos + 5;
-				break;
-
-			case 3:
+			} else {
 				/* partial body length not supported */
 				_alpm_log(handle, ALPM_LOG_ERROR,
-					_("%s: unsupported signature format"), identifier);
+					_("%s: unsupported signature format\n"), identifier);
 				return -1;
+			}
+		} else {
+			/* old package format */
+			switch(sig[pos] & 0x03) {
+				case 0:
+					if(length_check(len, pos, 2, handle, identifier) != 0) {
+						return -1;
+					}
+					blen = sig[pos + 1];
+					pos = pos + 2;
+					break;
+
+				case 1:
+					if(length_check(len, pos, 3, handle, identifier) != 0) {
+						return -1;
+					}
+					blen = (sig[pos + 1] << 8) | sig[pos + 2];
+					pos = pos + 3;
+					break;
+
+				case 2:
+					if(length_check(len, pos, 5, handle, identifier) != 0) {
+						return -1;
+					}
+					blen = (sig[pos + 1] << 24) | (sig[pos + 2] << 16) | (sig[pos + 3] << 8) | sig[pos + 4];
+					pos = pos + 5;
+					break;
+
+				case 3:
+					/* partial body length not supported */
+					_alpm_log(handle, ALPM_LOG_ERROR,
+						_("%s: unsupported signature format\n"), identifier);
+					return -1;
+			}
 		}
 
 		if(sig[pos] != 4) {
 			/* only support version 4 signature packet format */
 			_alpm_log(handle, ALPM_LOG_ERROR,
-					_("%s: unsupported signature format"), identifier);
+					_("%s: unsupported signature format\n"), identifier);
 			return -1;
 		}
-
 		if(sig[pos + 1] != 0x00) {
 			/* not a signature of a binary document */
 			_alpm_log(handle, ALPM_LOG_ERROR,
-					_("%s: signature format error"), identifier);
+					_("%s: signature format error\n"), identifier);
 			return -1;
 		}
-
 		pos = pos + 4;
 
+		/* pos got changed above, so an explicit check is necessary
+		 * check for 2 as that catches another some lines down */
+		if(length_check(len, pos, 2, handle, identifier)) {
+			return -1;
+		}
 		hlen = (sig[pos] << 8) | sig[pos + 1];
-		pos = pos + hlen + 2;
-
-		ulen = (sig[pos] << 8) | sig[pos + 1];
+		if(length_check(len, pos, hlen + 2, handle, identifier) != 0) {
+			return -1;
+		}
 		pos = pos + 2;
 
-		spos = pos;
-
-		while(spos < pos + ulen) {
-			if(sig[spos] < 192) {
-				slen = sig[spos];
-				spos = spos + 1;
-			} else if(sig[spos] < 255) {
-				slen = (sig[spos] << 8) | sig[spos + 1];
-				spos = spos + 2;
-			} else {
-				slen = (sig[spos + 1] << 24) | (sig[spos + 2] << 16) | (sig[spos + 3] << 8) | sig[spos + 4];
-				spos = spos + 5;
-			}
-
-			if(sig[spos] == 16) {
-				/* issuer key ID */
-				char key[17];
-				size_t i;
-				for (i = 0; i < 8; i++) {
-					sprintf(&key[i * 2], "%02X", sig[spos + i + 1]);
-				}
-				*keys = alpm_list_add(*keys, strdup(key));
-				break;
-			}
-
-			spos = spos + slen;
+		if(parse_subpacket(handle, identifier, sig, len, pos, hlen, keys) == -1) {
+			return -1;
 		}
+		pos = pos + hlen;
 
+		ulen = (sig[pos] << 8) | sig[pos + 1];
+		if(length_check(len, pos, ulen + 2, handle, identifier) != 0) {
+			return -1;
+		}
+		pos = pos + 2;
+
+		if(parse_subpacket(handle, identifier, sig, len, pos, ulen, keys) == -1) {
+			return -1;
+		}
 		pos = pos + (blen - hlen - 8);
 	}
 
 	return 0;
 }
-
-/* vim: set noet: */
