@@ -1,7 +1,7 @@
 /*
  *  sync.c
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
@@ -43,28 +43,29 @@
 #include "handle.h"
 #include "alpm.h"
 #include "dload.h"
-#include "delta.h"
 #include "remove.h"
 #include "diskspace.h"
 #include "signing.h"
 
+struct keyinfo_t {
+       char* uid;
+       char* keyid;
+};
+
+
 /** Check for new version of pkg in sync repos
  * (only the first occurrence is considered in sync)
  */
-alpm_pkg_t SYMEXPORT *alpm_sync_newversion(alpm_pkg_t *pkg, alpm_list_t *dbs_sync)
+alpm_pkg_t SYMEXPORT *alpm_sync_get_new_version(alpm_pkg_t *pkg, alpm_list_t *dbs_sync)
 {
 	alpm_list_t *i;
 	alpm_pkg_t *spkg = NULL;
 
 	ASSERT(pkg != NULL, return NULL);
-	pkg->handle->pm_errno = 0;
+	pkg->handle->pm_errno = ALPM_ERR_OK;
 
 	for(i = dbs_sync; !spkg && i; i = i->next) {
 		alpm_db_t *db = i->data;
-		if(!(db->usage & ALPM_DB_USAGE_SEARCH)) {
-			continue;
-		}
-
 		spkg = _alpm_db_get_pkgfromcache(db, pkg->name);
 	}
 
@@ -271,15 +272,27 @@ alpm_list_t SYMEXPORT *alpm_find_group_pkgs(alpm_list_t *dbs,
 		alpm_db_t *db = i->data;
 		alpm_group_t *grp = alpm_db_get_group(db, name);
 
-		if(!grp)
+		if(!grp) {
 			continue;
+		}
 
 		for(j = grp->packages; j; j = j->next) {
 			alpm_pkg_t *pkg = j->data;
+			alpm_trans_t *trans = db->handle->trans;
 
 			if(alpm_pkg_find(ignorelist, pkg->name)) {
 				continue;
 			}
+			if(trans != NULL && trans->flags & ALPM_TRANS_FLAG_NEEDED) {
+				alpm_pkg_t *local = _alpm_db_get_pkgfromcache(db->handle->db_local, pkg->name);
+				if(local && _alpm_pkg_compare_versions(pkg, local) == 0) {
+					/* with the NEEDED flag, packages up to date are not reinstalled */
+					_alpm_log(db->handle, ALPM_LOG_WARNING, _("%s-%s is up to date -- skipping\n"),
+							local->name, local->version);
+					ignorelist = alpm_list_add(ignorelist, pkg);
+					continue;
+				}
+			}
 			if(alpm_pkg_should_ignore(db->handle, pkg)) {
 				alpm_question_install_ignorepkg_t question = {
 					.type = ALPM_QUESTION_INSTALL_IGNOREPKG,
@@ -288,8 +301,9 @@ alpm_list_t SYMEXPORT *alpm_find_group_pkgs(alpm_list_t *dbs,
 				};
 				ignorelist = alpm_list_add(ignorelist, pkg);
 				QUESTION(db->handle, &question);
-				if(!question.install)
+				if(!question.install) {
 					continue;
+				}
 			}
 			if(!alpm_pkg_find(pkgs, pkg->name)) {
 				pkgs = alpm_list_add(pkgs, pkg);
@@ -342,21 +356,6 @@ static int compute_download_size(alpm_pkg_t *newpkg)
 
 		/* tell the caller that we have a partial */
 		ret = 1;
-	} else if(handle->deltaratio > 0.0) {
-		off_t dltsize;
-
-		dltsize = _alpm_shortest_delta_path(handle, newpkg->deltas,
-				newpkg->filename, &newpkg->delta_path);
-
-		if(newpkg->delta_path && (dltsize < newpkg->size * handle->deltaratio)) {
-			_alpm_log(handle, ALPM_LOG_DEBUG, "using delta size\n");
-			size = dltsize;
-		} else {
-			_alpm_log(handle, ALPM_LOG_DEBUG, "using package size\n");
-			size = newpkg->size;
-			alpm_list_free(newpkg->delta_path);
-			newpkg->delta_path = NULL;
-		}
 	} else {
 		size = newpkg->size;
 	}
@@ -460,7 +459,7 @@ int _alpm_sync_prepare(alpm_handle_t *handle, alpm_list_t **data)
 				   transaction. The packages will be removed from the actual
 				   transaction when the transaction packages are replaced with a
 				   dependency-reordered list below */
-				handle->pm_errno = 0;
+				handle->pm_errno = ALPM_ERR_OK;
 				if(data) {
 					alpm_list_free_inner(*data,
 							(alpm_list_fn_free)alpm_depmissing_free);
@@ -687,118 +686,6 @@ off_t SYMEXPORT alpm_pkg_download_size(alpm_pkg_t *newpkg)
 	return newpkg->download_size;
 }
 
-static int endswith(const char *filename, const char *extension)
-{
-	const char *s = filename + strlen(filename) - strlen(extension);
-	return strcmp(s, extension) == 0;
-}
-
-/** Applies delta files to create an upgraded package file.
- *
- * All intermediate files are deleted, leaving only the starting and
- * ending package files.
- *
- * @param handle the context handle
- *
- * @return 0 if all delta files were able to be applied, 1 otherwise.
- */
-static int apply_deltas(alpm_handle_t *handle)
-{
-	alpm_list_t *i;
-	size_t deltas_found = 0;
-	int ret = 0;
-	const char *cachedir = _alpm_filecache_setup(handle);
-	alpm_trans_t *trans = handle->trans;
-	alpm_event_delta_patch_t event;
-
-	for(i = trans->add; i; i = i->next) {
-		alpm_pkg_t *spkg = i->data;
-		alpm_list_t *delta_path = spkg->delta_path;
-		alpm_list_t *dlts = NULL;
-
-		if(!delta_path) {
-			continue;
-		}
-
-		if(!deltas_found) {
-			/* only show this if we actually have deltas to apply, and it is before
-			 * the very first one */
-			event.type = ALPM_EVENT_DELTA_PATCHES_START;
-			EVENT(handle, &event);
-			deltas_found = 1;
-		}
-
-		for(dlts = delta_path; dlts; dlts = dlts->next) {
-			alpm_delta_t *d = dlts->data;
-			char *delta, *from, *to;
-			char command[PATH_MAX];
-			size_t len = 0;
-
-			delta = _alpm_filecache_find(handle, d->delta);
-			/* the initial package might be in a different cachedir */
-			if(dlts == delta_path) {
-				from = _alpm_filecache_find(handle, d->from);
-			} else {
-				/* len = cachedir len + from len + '/' + null */
-				len = strlen(cachedir) + strlen(d->from) + 2;
-				MALLOC(from, len, free(delta); RET_ERR(handle, ALPM_ERR_MEMORY, 1));
-				snprintf(from, len, "%s/%s", cachedir, d->from);
-			}
-			len = strlen(cachedir) + strlen(d->to) + 2;
-			MALLOC(to, len, free(delta); free(from); RET_ERR(handle, ALPM_ERR_MEMORY, 1));
-			snprintf(to, len, "%s/%s", cachedir, d->to);
-
-			/* build the patch command */
-			if(endswith(to, ".gz")) {
-				/* special handling for gzip : we disable timestamp with -n option */
-				snprintf(command, PATH_MAX, "xdelta3 -d -q -R -c -s %s %s | gzip -n > %s", from, delta, to);
-			} else {
-				snprintf(command, PATH_MAX, "xdelta3 -d -q -s %s %s %s", from, delta, to);
-			}
-
-			_alpm_log(handle, ALPM_LOG_DEBUG, "command: %s\n", command);
-
-			event.type = ALPM_EVENT_DELTA_PATCH_START;
-			event.delta = d;
-			EVENT(handle, &event);
-
-			int retval = system(command);
-			if(retval == 0) {
-				event.type = ALPM_EVENT_DELTA_PATCH_DONE;
-				EVENT(handle, &event);
-
-				/* delete the delta file */
-				unlink(delta);
-
-				/* Delete the 'from' package but only if it is an intermediate
-				 * package. The starting 'from' package should be kept, just
-				 * as if deltas were not used. */
-				if(dlts != delta_path) {
-					unlink(from);
-				}
-			}
-			FREE(from);
-			FREE(to);
-			FREE(delta);
-
-			if(retval != 0) {
-				/* one delta failed for this package, cancel the remaining ones */
-				event.type = ALPM_EVENT_DELTA_PATCH_FAILED;
-				EVENT(handle, &event);
-				handle->pm_errno = ALPM_ERR_DLT_PATCHFAILED;
-				ret = 1;
-				break;
-			}
-		}
-	}
-	if(deltas_found) {
-		event.type = ALPM_EVENT_DELTA_PATCHES_DONE;
-		EVENT(handle, &event);
-	}
-
-	return ret;
-}
-
 /**
  * Prompts to delete the file now that we know it is invalid.
  * @param handle the context handle
@@ -823,44 +710,6 @@ static int prompt_to_delete(alpm_handle_t *handle, const char *filepath,
 	return question.remove;
 }
 
-static int validate_deltas(alpm_handle_t *handle, alpm_list_t *deltas)
-{
-	alpm_list_t *i, *errors = NULL;
-	alpm_event_t event;
-
-	if(!deltas) {
-		return 0;
-	}
-
-	/* Check integrity of deltas */
-	event.type = ALPM_EVENT_DELTA_INTEGRITY_START;
-	EVENT(handle, &event);
-	for(i = deltas; i; i = i->next) {
-		alpm_delta_t *d = i->data;
-		char *filepath = _alpm_filecache_find(handle, d->delta);
-
-		if(_alpm_test_checksum(filepath, d->delta_md5, ALPM_PKG_VALIDATION_MD5SUM)) {
-			errors = alpm_list_add(errors, filepath);
-		} else {
-			FREE(filepath);
-		}
-	}
-	event.type = ALPM_EVENT_DELTA_INTEGRITY_DONE;
-	EVENT(handle, &event);
-
-	if(errors) {
-		for(i = errors; i; i = i->next) {
-			char *filepath = i->data;
-			prompt_to_delete(handle, filepath, ALPM_ERR_DLT_INVALID);
-			FREE(filepath);
-		}
-		alpm_list_free(errors);
-		handle->pm_errno = ALPM_ERR_DLT_INVALID;
-		return -1;
-	}
-	return 0;
-}
-
 static struct dload_payload *build_payload(alpm_handle_t *handle,
 		const char *filename, size_t size, alpm_list_t *servers)
 {
@@ -873,7 +722,7 @@ static struct dload_payload *build_payload(alpm_handle_t *handle,
 		return payload;
 }
 
-static int find_dl_candidates(alpm_db_t *repo, alpm_list_t **files, alpm_list_t **deltas)
+static int find_dl_candidates(alpm_db_t *repo, alpm_list_t **files)
 {
 	alpm_list_t *i;
 	alpm_handle_t *handle = repo->handle;
@@ -882,8 +731,6 @@ static int find_dl_candidates(alpm_db_t *repo, alpm_list_t **files, alpm_list_t
 		alpm_pkg_t *spkg = i->data;
 
 		if(spkg->origin != ALPM_PKG_FROM_FILE && repo == spkg->origin_data.db) {
-			alpm_list_t *delta_path = spkg->delta_path;
-
 			if(!repo->servers) {
 				handle->pm_errno = ALPM_ERR_SERVER_NONE;
 				_alpm_log(handle, ALPM_LOG_ERROR, "%s: %s\n",
@@ -891,22 +738,7 @@ static int find_dl_candidates(alpm_db_t *repo, alpm_list_t **files, alpm_list_t
 				return 1;
 			}
 
-			if(delta_path) {
-				/* using deltas */
-				alpm_list_t *dlts;
-				for(dlts = delta_path; dlts; dlts = dlts->next) {
-					alpm_delta_t *delta = dlts->data;
-					if(delta->download_size != 0) {
-						struct dload_payload *payload = build_payload(
-								handle, delta->delta, delta->delta_size, repo->servers);
-						ASSERT(payload, return -1);
-						*files = alpm_list_add(*files, payload);
-					}
-					/* keep a list of all the delta files for md5sums */
-					*deltas = alpm_list_add(*deltas, delta);
-				}
-
-			} else if(spkg->download_size != 0) {
+			if(spkg->download_size != 0) {
 				struct dload_payload *payload;
 				ASSERT(spkg->filename != NULL, RET_ERR(handle, ALPM_ERR_PKG_INVALID_NAME, -1));
 				payload = build_payload(handle, spkg->filename, spkg->size, repo->servers);
@@ -946,9 +778,7 @@ static int download_single_file(alpm_handle_t *handle, struct dload_payload *pay
 			EVENT(handle, &event);
 			return 0;
 		}
-
-		FREE(payload->fileurl);
-		payload->unlink_on_fail = 0;
+		_alpm_dload_payload_reset_for_retry(payload);
 	}
 
 	event.type = ALPM_EVENT_PKGDOWNLOAD_FAILED;
@@ -956,7 +786,7 @@ static int download_single_file(alpm_handle_t *handle, struct dload_payload *pay
 	return -1;
 }
 
-static int download_files(alpm_handle_t *handle, alpm_list_t **deltas)
+static int download_files(alpm_handle_t *handle)
 {
 	const char *cachedir;
 	alpm_list_t *i, *files = NULL;
@@ -980,7 +810,7 @@ static int download_files(alpm_handle_t *handle, alpm_list_t **deltas)
 	}
 
 	for(i = handle->dbs_sync; i; i = i->next) {
-		errors += find_dl_candidates(i->data, &files, deltas);
+		errors += find_dl_candidates(i->data, &files);
 	}
 
 	if(files) {
@@ -1048,6 +878,7 @@ static int check_keyring(alpm_handle_t *handle)
 	size_t current = 0, numtargs;
 	alpm_list_t *i, *errors = NULL;
 	alpm_event_t event;
+	struct keyinfo_t *keyinfo;
 
 	event.type = ALPM_EVENT_KEYRING_START;
 	EVENT(handle, &event);
@@ -1056,7 +887,7 @@ static int check_keyring(alpm_handle_t *handle)
 
 	for(i = handle->trans->add; i; i = i->next, current++) {
 		alpm_pkg_t *pkg = i->data;
-		alpm_siglevel_t level;
+		int level;
 
 		int percent = (current * 100) / numtargs;
 		PROGRESS(handle, ALPM_PROGRESS_KEYRING_START, "", percent,
@@ -1081,7 +912,13 @@ static int check_keyring(alpm_handle_t *handle)
 						char *key = k->data;
 						if(!alpm_list_find_str(errors, key) &&
 								_alpm_key_in_keychain(handle, key) == 0) {
-							errors = alpm_list_add(errors, strdup(key));
+							keyinfo = malloc(sizeof(struct keyinfo_t));
+							if(!keyinfo) {
+								break;
+							}
+							keyinfo->uid = strdup(pkg->packager);
+							keyinfo->keyid = strdup(key);
+							errors = alpm_list_add(errors, keyinfo);
 						}
 					}
 					FREELIST(keys);
@@ -1102,10 +939,13 @@ static int check_keyring(alpm_handle_t *handle)
 		int fail = 0;
 		alpm_list_t *k;
 		for(k = errors; k; k = k->next) {
-			char *key = k->data;
-			if(_alpm_key_import(handle, key) == -1) {
+			keyinfo = k->data;
+				if(_alpm_key_import(handle, keyinfo->uid, keyinfo->keyid) == -1) {
 				fail = 1;
 			}
+			free(keyinfo->uid);
+			free(keyinfo->keyid);
+
 		}
 		event.type = ALPM_EVENT_KEY_DOWNLOAD_DONE;
 		EVENT(handle, &event);
@@ -1126,8 +966,8 @@ static int check_validity(alpm_handle_t *handle,
 		alpm_pkg_t *pkg;
 		char *path;
 		alpm_siglist_t *siglist;
-		alpm_siglevel_t level;
-		alpm_pkgvalidation_t validation;
+		int siglevel;
+		int validation;
 		alpm_errno_t error;
 	};
 	size_t current = 0;
@@ -1151,10 +991,10 @@ static int check_validity(alpm_handle_t *handle,
 
 		current_bytes += v.pkg->size;
 		v.path = _alpm_filecache_find(handle, v.pkg->filename);
-		v.level = alpm_db_get_siglevel(alpm_pkg_get_db(v.pkg));
+		v.siglevel = alpm_db_get_siglevel(alpm_pkg_get_db(v.pkg));
 
 		if(_alpm_pkg_validate_internal(handle, v.path, v.pkg,
-					v.level, &v.siglist, &v.validation) == -1) {
+					v.siglevel, &v.siglist, &v.validation) == -1) {
 			struct validity *invalid;
 			v.error = handle->pm_errno;
 			MALLOC(invalid, sizeof(struct validity), return -1);
@@ -1176,17 +1016,28 @@ static int check_validity(alpm_handle_t *handle,
 	if(errors) {
 		for(i = errors; i; i = i->next) {
 			struct validity *v = i->data;
-			if(v->error == ALPM_ERR_PKG_MISSING_SIG) {
-				_alpm_log(handle, ALPM_LOG_ERROR,
-						_("%s: missing required signature\n"), v->pkg->name);
-			} else if(v->error == ALPM_ERR_PKG_INVALID_SIG) {
-				_alpm_process_siglist(handle, v->pkg->name, v->siglist,
-						v->level & ALPM_SIG_PACKAGE_OPTIONAL,
-						v->level & ALPM_SIG_PACKAGE_MARGINAL_OK,
-						v->level & ALPM_SIG_PACKAGE_UNKNOWN_OK);
-				prompt_to_delete(handle, v->path, v->error);
-			} else if(v->error == ALPM_ERR_PKG_INVALID_CHECKSUM) {
-				prompt_to_delete(handle, v->path, v->error);
+			switch(v->error) {
+				case ALPM_ERR_PKG_MISSING_SIG:
+					_alpm_log(handle, ALPM_LOG_ERROR,
+							_("%s: missing required signature\n"), v->pkg->name);
+					break;
+				case ALPM_ERR_PKG_INVALID_SIG:
+					_alpm_process_siglist(handle, v->pkg->name, v->siglist,
+							v->siglevel & ALPM_SIG_PACKAGE_OPTIONAL,
+							v->siglevel & ALPM_SIG_PACKAGE_MARGINAL_OK,
+							v->siglevel & ALPM_SIG_PACKAGE_UNKNOWN_OK);
+					/* fallthrough */
+				case ALPM_ERR_PKG_INVALID_CHECKSUM:
+					prompt_to_delete(handle, v->path, v->error);
+					break;
+				case ALPM_ERR_PKG_NOT_FOUND:
+				case ALPM_ERR_BADPERMS:
+				case ALPM_ERR_PKG_OPEN:
+					_alpm_log(handle, ALPM_LOG_ERROR, _("failed to read file %s: %s\n"), v->path, alpm_strerror(v->error));
+					break;
+				default:
+					/* ignore */
+					break;
 			}
 			alpm_siglist_cleanup(v->siglist);
 			free(v->siglist);
@@ -1195,7 +1046,7 @@ static int check_validity(alpm_handle_t *handle,
 		}
 		alpm_list_free(errors);
 
-		if(!handle->pm_errno) {
+		if(handle->pm_errno == ALPM_ERR_OK) {
 			RET_ERR(handle, ALPM_ERR_PKG_INVALID, -1);
 		}
 		return -1;
@@ -1280,7 +1131,7 @@ static int load_packages(alpm_handle_t *handle, alpm_list_t **data,
 	EVENT(handle, &event);
 
 	if(errors) {
-		if(!handle->pm_errno) {
+		if(handle->pm_errno == ALPM_ERR_OK) {
 			RET_ERR(handle, ALPM_ERR_PKG_INVALID, -1);
 		}
 		return -1;
@@ -1291,24 +1142,12 @@ static int load_packages(alpm_handle_t *handle, alpm_list_t **data,
 
 int _alpm_sync_load(alpm_handle_t *handle, alpm_list_t **data)
 {
-	alpm_list_t *i, *deltas = NULL;
+	alpm_list_t *i;
 	size_t total = 0;
 	uint64_t total_bytes = 0;
 	alpm_trans_t *trans = handle->trans;
 
-	if(download_files(handle, &deltas)) {
-		alpm_list_free(deltas);
-		return -1;
-	}
-
-	if(validate_deltas(handle, deltas)) {
-		alpm_list_free(deltas);
-		return -1;
-	}
-	alpm_list_free(deltas);
-
-	/* Use the deltas to generate the packages */
-	if(apply_deltas(handle)) {
+	if(download_files(handle)) {
 		return -1;
 	}
 
@@ -1417,5 +1256,3 @@ int _alpm_sync_commit(alpm_handle_t *handle)
 
 	return 0;
 }
-
-/* vim: set noet: */
