@@ -1,7 +1,7 @@
 /*
  *  conf.c
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,13 +29,13 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/utsname.h> /* uname */
+#include <sys/wait.h>
 #include <unistd.h>
 
 /* pacman */
 #include "conf.h"
 #include "ini.h"
 #include "util.h"
-#include "pacman.h"
 #include "callback.h"
 
 /* global config variable */
@@ -104,7 +104,6 @@ config_t *config_new(void)
 	newconfig->op = PM_OP_MAIN;
 	newconfig->logmask = ALPM_LOG_ERROR | ALPM_LOG_WARNING;
 	newconfig->configfile = strdup(CONFFILE);
-	newconfig->deltaratio = 0.0;
 	if(alpm_capabilities() & ALPM_CAPABILITY_SIGNATURES) {
 		newconfig->siglevel = ALPM_SIG_PACKAGE | ALPM_SIG_PACKAGE_OPTIONAL |
 			ALPM_SIG_DATABASE | ALPM_SIG_DATABASE_OPTIONAL;
@@ -143,6 +142,7 @@ int config_free(config_t *oldconfig)
 	FREELIST(oldconfig->assumeinstalled);
 	FREELIST(oldconfig->noupgrade);
 	FREELIST(oldconfig->noextract);
+	FREELIST(oldconfig->overwrite_files);
 	free(oldconfig->configfile);
 	free(oldconfig->rootdir);
 	free(oldconfig->dbpath);
@@ -153,6 +153,7 @@ int config_free(config_t *oldconfig)
 	free(oldconfig->xfercommand);
 	free(oldconfig->print_format);
 	free(oldconfig->arch);
+	wordsplit_free(oldconfig->xfercommand_argv);
 	free(oldconfig);
 
 	return 0;
@@ -202,18 +203,86 @@ static char *get_tempfile(const char *path, const char *filename)
 	return tempfile;
 }
 
+/* system()/exec() hybrid function allowing exec()-style direct execution
+ * of a command with the simplicity of system()
+ * - not thread-safe
+ * - errno may be set by fork(), pipe(), or execvp()
+ */
+static int systemvp(const char *file, char *const argv[])
+{
+	int pid, err = 0, ret = -1, err_fd[2];
+	sigset_t oldblock;
+	struct sigaction sa_ign = { .sa_handler = SIG_IGN }, oldint, oldquit;
+
+	if(pipe(err_fd) != 0) {
+		return -1;
+	}
+
+	sigaction(SIGINT, &sa_ign, &oldint);
+	sigaction(SIGQUIT, &sa_ign, &oldquit);
+	sigaddset(&sa_ign.sa_mask, SIGCHLD);
+	sigprocmask(SIG_BLOCK, &sa_ign.sa_mask, &oldblock);
+
+	pid = fork();
+
+	/* child */
+	if(pid == 0) {
+		close(err_fd[0]);
+		fcntl(err_fd[1], F_SETFD, FD_CLOEXEC);
+
+		/* restore signal handling for the child to inherit */
+		sigaction(SIGINT, &oldint, NULL);
+		sigaction(SIGQUIT, &oldquit, NULL);
+		sigprocmask(SIG_SETMASK, &oldblock, NULL);
+
+		execvp(file, argv);
+
+		/* execvp failed, pass the error back to the parent */
+		while(write(err_fd[1], &errno, sizeof(errno)) == -1 && errno == EINTR);
+		_Exit(127);
+	}
+
+	/* parent */
+	close(err_fd[1]);
+
+	if(pid != -1)  {
+		int wret;
+		while((wret = waitpid(pid, &ret, 0)) == -1 && errno == EINTR);
+		if(wret > 0) {
+			while(read(err_fd[0], &err, sizeof(err)) == -1 && errno == EINTR);
+		}
+	} else {
+		/* fork failed, make sure errno is preserved after cleanup */
+		err = errno;
+	}
+
+	close(err_fd[0]);
+
+	sigaction(SIGINT, &oldint, NULL);
+	sigaction(SIGQUIT, &oldquit, NULL);
+	sigprocmask(SIG_SETMASK, &oldblock, NULL);
+
+	if(err) {
+		errno = err;
+		ret = -1;
+	}
+
+	return ret;
+}
+
 /** External fetch callback */
 static int download_with_xfercommand(const char *url, const char *localpath,
 		int force)
 {
 	int ret = 0, retval;
 	int usepart = 0;
-	int cwdfd;
+	int cwdfd = -1;
 	struct stat st;
-	char *parsedcmd, *tempcmd;
 	char *destfile, *tempfile, *filename;
+	const char **argv;
+	size_t i;
 
-	if(!config->xfercommand) {
+	if(!config->xfercommand_argv) {
 		return -1;
 	}
 
@@ -231,17 +300,26 @@ static int download_with_xfercommand(const char *url, const char *localpath,
 		unlink(destfile);
 	}
 
-	tempcmd = strdup(config->xfercommand);
-	/* replace all occurrences of %o with fn.part */
-	if(strstr(tempcmd, "%o")) {
-		usepart = 1;
-		parsedcmd = strreplace(tempcmd, "%o", tempfile);
-		free(tempcmd);
-		tempcmd = parsedcmd;
+	if((argv = calloc(config->xfercommand_argc + 1, sizeof(char*))) == NULL) {
+		size_t bytes = (config->xfercommand_argc + 1) * sizeof(char*);
+		pm_printf(ALPM_LOG_ERROR,
+				_n("malloc failure: could not allocate %zu byte\n",
+				   "malloc failure: could not allocate %zu bytes\n",
+					 bytes),
+				bytes);
+		goto cleanup;
+	}
+
+	for(i = 0; i <= config->xfercommand_argc; i++) {
+		const char *val = config->xfercommand_argv[i];
+		if(val && strcmp(val, "%o") == 0) {
+			usepart = 1;
+			val = tempfile;
+		} else if(val && strcmp(val, "%u") == 0) {
+			val = url;
+		}
+		argv[i] = val;
 	}
-	/* replace all occurrences of %u with the download URL */
-	parsedcmd = strreplace(tempcmd, "%u", url);
-	free(tempcmd);
 
 	/* save the cwd so we can restore it later */
 	do {
@@ -257,9 +335,15 @@ static int download_with_xfercommand(const char *url, const char *localpath,
 		ret = -1;
 		goto cleanup;
 	}
-	/* execute the parsed command via /bin/sh -c */
-	pm_printf(ALPM_LOG_DEBUG, "running command: %s\n", parsedcmd);
-	retval = system(parsedcmd);
+
+	if(config->logmask & ALPM_LOG_DEBUG) {
+		char *cmd = arg_to_string(config->xfercommand_argc, (char**)argv);
+		if(cmd) {
+			pm_printf(ALPM_LOG_DEBUG, "running command: %s\n", cmd);
+			free(cmd);
+		}
+	}
+	retval = systemvp(argv[0], (char**)argv);
 
 	if(retval == -1) {
 		pm_printf(ALPM_LOG_WARNING, _("running XferCommand: fork failed!\n"));
@@ -297,7 +381,7 @@ cleanup:
 	}
 	free(destfile);
 	free(tempfile);
-	free(parsedcmd);
+	free(argv);
 
 	return ret;
 }
@@ -325,10 +409,10 @@ int config_set_arch(const char *arch)
  * @param linenum current line number in file
  * @return 0 on success, 1 on any parsing error
  */
-static int process_siglevel(alpm_list_t *values, alpm_siglevel_t *storage,
-		alpm_siglevel_t *storage_mask, const char *file, int linenum)
+static int process_siglevel(alpm_list_t *values, int *storage,
+		int *storage_mask, const char *file, int linenum)
 {
-	alpm_siglevel_t level = *storage, mask = *storage_mask;
+	int level = *storage, mask = *storage_mask;
 	alpm_list_t *i;
 	int ret = 0;
 
@@ -421,13 +505,12 @@ static int process_siglevel(alpm_list_t *values, alpm_siglevel_t *storage,
 }
 
 /**
- * Merge the package entires of two signature verification levels.
+ * Merge the package entries of two signature verification levels.
  * @param base initial siglevel
- * @param over overridden siglevel
+ * @param over overriding siglevel
  * @return merged siglevel
  */
-static alpm_siglevel_t merge_siglevel(alpm_siglevel_t base,
-		alpm_siglevel_t over, alpm_siglevel_t mask)
+static int merge_siglevel(int base, int over, int mask)
 {
 	return mask ? (over & mask) | (base & ~mask) : over;
 }
@@ -486,9 +569,6 @@ static int _parse_options(const char *key, char *value,
 		} else if(strcmp(key, "VerbosePkgLists") == 0) {
 			config->verbosepkglists = 1;
 			pm_printf(ALPM_LOG_DEBUG, "config: verbosepkglists\n");
-		} else if(strcmp(key, "UseDelta") == 0) {
-			config->deltaratio = 0.7;
-			pm_printf(ALPM_LOG_DEBUG, "config: usedelta (default 0.7)\n");
 		} else if(strcmp(key, "TotalDownload") == 0) {
 			config->totaldownload = 1;
 			pm_printf(ALPM_LOG_DEBUG, "config: totaldownload\n");
@@ -499,6 +579,8 @@ static int _parse_options(const char *key, char *value,
 				config->color = isatty(fileno(stdout)) ? PM_COLOR_ON : PM_COLOR_OFF;
 				enable_colors(config->color);
 			}
+		} else if(strcmp(key, "DisableDownloadTimeout") == 0) {
+			config->disable_dl_timeout = 1;
 		} else {
 			pm_printf(ALPM_LOG_WARNING,
 					_("config file %s, line %d: directive '%s' in section '%s' not recognized.\n"),
@@ -524,26 +606,6 @@ static int _parse_options(const char *key, char *value,
 			if(!config->arch) {
 				config_set_arch(value);
 			}
-		} else if(strcmp(key, "UseDelta") == 0) {
-			double ratio;
-			char *endptr;
-			const char *oldlocale;
-
-			/* set the locale to 'C' for consistent decimal parsing (0.7 and never
-			 * 0,7) from config files, then restore old setting when we are done */
-			oldlocale = setlocale(LC_NUMERIC, NULL);
-			setlocale(LC_NUMERIC, "C");
-			ratio = strtod(value, &endptr);
-			setlocale(LC_NUMERIC, oldlocale);
-
-			if(*endptr != '\0' || ratio < 0.0 || ratio > 2.0) {
-				pm_printf(ALPM_LOG_ERROR,
-						_("config file %s, line %d: invalid value for '%s' : '%s'\n"),
-						file, linenum, "UseDelta", value);
-				return 1;
-			}
-			config->deltaratio = ratio;
-			pm_printf(ALPM_LOG_DEBUG, "config: usedelta = %f\n", ratio);
 		} else if(strcmp(key, "DBPath") == 0) {
 			/* don't overwrite a path specified on the command line */
 			if(!config->dbpath) {
@@ -567,6 +629,17 @@ static int _parse_options(const char *key, char *value,
 				pm_printf(ALPM_LOG_DEBUG, "config: logfile: %s\n", value);
 			}
 		} else if(strcmp(key, "XferCommand") == 0) {
+			char **c;
+			if((config->xfercommand_argv = wordsplit(value)) == NULL) {
+				pm_printf(ALPM_LOG_WARNING,
+						_("config file %s, line %d: invalid value for '%s' : '%s'\n"),
+						file, linenum, "XferCommand", value);
+				return 1;
+			}
+			config->xfercommand_argc = 0;
+			for(c = config->xfercommand_argv; *c; c++) {
+				config->xfercommand_argc++;
+			}
 			config->xfercommand = strdup(value);
 			pm_printf(ALPM_LOG_DEBUG, "config: xfercommand: %s\n", value);
 		} else if(strcmp(key, "CleanMethod") == 0) {
@@ -614,37 +687,39 @@ static int _parse_options(const char *key, char *value,
 	return 0;
 }
 
-static int _add_mirror(alpm_db_t *db, char *value)
+static char *replace_server_vars(config_t *c, config_repo_t *r, const char *s)
 {
-	const char *dbname = alpm_db_get_name(db);
-	/* let's attempt a replacement for the current repo */
-	char *temp = strreplace(value, "$repo", dbname);
-	/* let's attempt a replacement for the arch */
-	const char *arch = config->arch;
-	char *server;
-	if(arch) {
-		server = strreplace(temp, "$arch", arch);
-		free(temp);
-	} else {
-		if(strstr(temp, "$arch")) {
-			free(temp);
-			pm_printf(ALPM_LOG_ERROR,
-					_("mirror '%s' contains the '%s' variable, but no '%s' is defined.\n"),
-					value, "$arch", "Architecture");
-			return 1;
-		}
-		server = temp;
+	if(c->arch == NULL && strstr(s, "$arch")) {
+		pm_printf(ALPM_LOG_ERROR,
+				_("mirror '%s' contains the '%s' variable, but no '%s' is defined.\n"),
+				s, "$arch", "Architecture");
+		return NULL;
 	}
 
-	if(alpm_db_add_server(db, server) != 0) {
+	if(c->arch) {
+		char *temp, *replaced;
+
+		replaced = strreplace(s, "$arch", c->arch);
+
+		temp = replaced;
+		replaced = strreplace(temp, "$repo", r->name);
+		free(temp);
+
+		return replaced;
+	} else {
+		return strreplace(s, "$repo", r->name);
+	}
+}
+
+static int _add_mirror(alpm_db_t *db, char *value)
+{
+	if(alpm_db_add_server(db, value) != 0) {
 		/* pm_errno is set by alpm_db_setserver */
 		pm_printf(ALPM_LOG_ERROR, _("could not add server URL to database '%s': %s (%s)\n"),
-				dbname, server, alpm_strerror(alpm_errno(config->handle)));
-		free(server);
+				alpm_db_get_name(db), value, alpm_strerror(alpm_errno(config->handle)));
 		return 1;
 	}
 
-	free(server);
 	return 0;
 }
 
@@ -653,9 +728,6 @@ static int register_repo(config_repo_t *repo)
 	alpm_list_t *i;
 	alpm_db_t *db;
 
-	repo->siglevel = merge_siglevel(config->siglevel,
-			repo->siglevel, repo->siglevel_mask);
-
 	db = alpm_register_syncdb(config->handle, repo->name, repo->siglevel);
 	if(db == NULL) {
 		pm_printf(ALPM_LOG_ERROR, _("could not register '%s' database (%s)\n"),
@@ -663,18 +735,12 @@ static int register_repo(config_repo_t *repo)
 		return 1;
 	}
 
-	pm_printf(ALPM_LOG_DEBUG,
-			"setting usage of %d for %s repository\n",
-			repo->usage == 0 ? ALPM_DB_USAGE_ALL : repo->usage,
-			repo->name);
-	alpm_db_set_usage(db, repo->usage == 0 ? ALPM_DB_USAGE_ALL : repo->usage);
+	pm_printf(ALPM_LOG_DEBUG, "setting usage of %d for %s repository\n",
+			repo->usage, repo->name);
+	alpm_db_set_usage(db, repo->usage);
 
 	for(i = repo->servers; i; i = alpm_list_next(i)) {
-		char *value = i->data;
-		if(_add_mirror(db, value) != 0) {
-			pm_printf(ALPM_LOG_ERROR,
-					_("could not add mirror '%s' to database '%s' (%s)\n"),
-					value, repo->name, alpm_strerror(alpm_errno(config->handle)));
+		if(_add_mirror(db, i->data) != 0) {
 			return 1;
 		}
 	}
@@ -697,25 +763,6 @@ static int setup_libalpm(void)
 
 	pm_printf(ALPM_LOG_DEBUG, "setup_libalpm called\n");
 
-	/* Configure root path first. If it is set and dbpath/logfile were not
-	 * set, then set those as well to reside under the root. */
-	if(config->rootdir) {
-		char path[PATH_MAX];
-		if(!config->dbpath) {
-			snprintf(path, PATH_MAX, "%s/%s", config->rootdir, DBPATH + 1);
-			config->dbpath = strdup(path);
-		}
-		if(!config->logfile) {
-			snprintf(path, PATH_MAX, "%s/%s", config->rootdir, LOGFILE + 1);
-			config->logfile = strdup(path);
-		}
-	} else {
-		config->rootdir = strdup(ROOTDIR);
-		if(!config->dbpath) {
-			config->dbpath = strdup(DBPATH);
-		}
-	}
-
 	/* initialize library */
 	handle = alpm_initialize(config->rootdir, config->dbpath, &err);
 	if(!handle) {
@@ -738,7 +785,6 @@ static int setup_libalpm(void)
 		alpm_option_set_dbext(handle, ".files");
 	}
 
-	config->logfile = config->logfile ? config->logfile : strdup(LOGFILE);
 	ret = alpm_option_set_logfile(handle, config->logfile);
 	if(ret != 0) {
 		pm_printf(ALPM_LOG_ERROR, _("problem setting logfile '%s' (%s)\n"),
@@ -748,7 +794,6 @@ static int setup_libalpm(void)
 
 	/* Set GnuPG's home directory. This is not relative to rootdir, even if
 	 * rootdir is defined. Reasoning: gpgdir contains configuration data. */
-	config->gpgdir = config->gpgdir ? config->gpgdir : strdup(GPGDIR);
 	ret = alpm_option_set_gpgdir(handle, config->gpgdir);
 	if(ret != 0) {
 		pm_printf(ALPM_LOG_ERROR, _("problem setting gpgdir '%s' (%s)\n"),
@@ -758,37 +803,21 @@ static int setup_libalpm(void)
 
 	/* Set user hook directory. This is not relative to rootdir, even if
 	 * rootdir is defined. Reasoning: hookdir contains configuration data. */
-	if(config->hookdirs == NULL) {
-		if((ret = alpm_option_add_hookdir(handle, HOOKDIR)) != 0) {
+	/* add hook directories 1-by-1 to avoid overwriting the system directory */
+	for(i = config->hookdirs; i; i = alpm_list_next(i)) {
+		if((ret = alpm_option_add_hookdir(handle, i->data)) != 0) {
 			pm_printf(ALPM_LOG_ERROR, _("problem adding hookdir '%s' (%s)\n"),
-					HOOKDIR, alpm_strerror(alpm_errno(handle)));
+					(char *) i->data, alpm_strerror(alpm_errno(handle)));
 			return ret;
 		}
-	} else {
-		/* add hook directories 1-by-1 to avoid overwriting the system directory */
-		for(i = config->hookdirs; i; i = alpm_list_next(i)) {
-			if((ret = alpm_option_add_hookdir(handle, i->data)) != 0) {
-				pm_printf(ALPM_LOG_ERROR, _("problem adding hookdir '%s' (%s)\n"),
-						(char *) i->data, alpm_strerror(alpm_errno(handle)));
-				return ret;
-			}
-		}
 	}
 
-	/* add a default cachedir if one wasn't specified */
-	if(config->cachedirs == NULL) {
-		alpm_option_add_cachedir(handle, CACHEDIR);
-	} else {
-		alpm_option_set_cachedirs(handle, config->cachedirs);
-	}
+	alpm_option_set_cachedirs(handle, config->cachedirs);
+
+	alpm_option_set_overwrite_files(handle, config->overwrite_files);
 
 	alpm_option_set_default_siglevel(handle, config->siglevel);
 
-	config->localfilesiglevel = merge_siglevel(config->siglevel,
-			config->localfilesiglevel, config->localfilesiglevel_mask);
-	config->remotefilesiglevel = merge_siglevel(config->siglevel,
-			config->remotefilesiglevel, config->remotefilesiglevel_mask);
-
 	alpm_option_set_local_file_siglevel(handle, config->localfilesiglevel);
 	alpm_option_set_remote_file_siglevel(handle, config->remotefilesiglevel);
 
@@ -809,13 +838,14 @@ static int setup_libalpm(void)
 	alpm_option_set_arch(handle, config->arch);
 	alpm_option_set_checkspace(handle, config->checkspace);
 	alpm_option_set_usesyslog(handle, config->usesyslog);
-	alpm_option_set_deltaratio(handle, config->deltaratio);
 
 	alpm_option_set_ignorepkgs(handle, config->ignorepkg);
 	alpm_option_set_ignoregroups(handle, config->ignoregrp);
 	alpm_option_set_noupgrades(handle, config->noupgrade);
 	alpm_option_set_noextracts(handle, config->noextract);
 
+	alpm_option_set_disable_dl_timeout(handle, config->disable_dl_timeout);
+
 	for(i = config->assumeinstalled; i; i = i->next) {
 		char *entry = i->data;
 		alpm_depend_t *dep = alpm_dep_from_string(entry);
@@ -845,11 +875,11 @@ struct section_t {
 	int depth;
 };
 
-static int process_usage(alpm_list_t *values, alpm_db_usage_t *usage,
+static int process_usage(alpm_list_t *values, int *usage,
 		const char *file, int linenum)
 {
 	alpm_list_t *i;
-	alpm_db_usage_t level = *usage;
+	int level = *usage;
 	int ret = 0;
 
 	for(i = values; i; i = i->next) {
@@ -885,28 +915,28 @@ static int _parse_repo(const char *key, char *value, const char *file,
 	int ret = 0;
 	config_repo_t *repo = section->repo;
 
+#define CHECK_VALUE(val) do { \
+	if(!val) { \
+		pm_printf(ALPM_LOG_ERROR, _("config file %s, line %d: directive '%s' needs a value\n"), \
+				file, line, key); \
+		return 1; \
+	} \
+} while(0)
+
 	if(strcmp(key, "Server") == 0) {
-		if(!value) {
-			pm_printf(ALPM_LOG_ERROR, _("config file %s, line %d: directive '%s' needs a value\n"),
-					file, line, key);
-			ret = 1;
-		} else {
-			repo->servers = alpm_list_add(repo->servers, strdup(value));
-		}
+		CHECK_VALUE(value);
+		repo->servers = alpm_list_add(repo->servers, strdup(value));
 	} else if(strcmp(key, "SigLevel") == 0) {
-		if(!value) {
-			pm_printf(ALPM_LOG_ERROR, _("config file %s, line %d: directive '%s' needs a value\n"),
-					file, line, key);
-		} else {
-			alpm_list_t *values = NULL;
-			setrepeatingoption(value, "SigLevel", &values);
-			if(values) {
-				ret = process_siglevel(values, &repo->siglevel,
-						&repo->siglevel_mask, file, line);
-				FREELIST(values);
-			}
+		CHECK_VALUE(value);
+		alpm_list_t *values = NULL;
+		setrepeatingoption(value, "SigLevel", &values);
+		if(values) {
+			ret = process_siglevel(values, &repo->siglevel,
+					&repo->siglevel_mask, file, line);
+			FREELIST(values);
 		}
 	} else if(strcmp(key, "Usage") == 0) {
+		CHECK_VALUE(value);
 		alpm_list_t *values = NULL;
 		setrepeatingoption(value, "Usage", &values);
 		if(values) {
@@ -922,6 +952,8 @@ static int _parse_repo(const char *key, char *value, const char *file,
 				file, line, key, repo->name);
 	}
 
+#undef CHECK_VALUE
+
 	return ret;
 }
 
@@ -1030,6 +1062,67 @@ static int _parse_directive(const char *file, int linenum, const char *name,
 	}
 }
 
+int setdefaults(config_t *c)
+{
+	alpm_list_t *i;
+
+#define SETDEFAULT(opt, val) if(!opt) { opt = val; if(!opt) { return -1; } }
+
+	if(c->rootdir) {
+		char path[PATH_MAX];
+		if(!c->dbpath) {
+			snprintf(path, PATH_MAX, "%s/%s", c->rootdir, &DBPATH[1]);
+			SETDEFAULT(c->dbpath, strdup(path));
+		}
+		if(!c->logfile) {
+			snprintf(path, PATH_MAX, "%s/%s", c->rootdir, &LOGFILE[1]);
+			SETDEFAULT(c->logfile, strdup(path));
+		}
+	} else {
+		SETDEFAULT(c->rootdir, strdup(ROOTDIR));
+		SETDEFAULT(c->dbpath, strdup(DBPATH));
+	}
+
+	SETDEFAULT(c->logfile, strdup(LOGFILE));
+	SETDEFAULT(c->gpgdir, strdup(GPGDIR));
+	SETDEFAULT(c->cachedirs, alpm_list_add(NULL, strdup(CACHEDIR)));
+	SETDEFAULT(c->hookdirs, alpm_list_add(NULL, strdup(HOOKDIR)));
+	SETDEFAULT(c->cleanmethod, PM_CLEAN_KEEPINST);
+
+	c->localfilesiglevel = merge_siglevel(c->siglevel,
+			c->localfilesiglevel, c->localfilesiglevel_mask);
+	c->remotefilesiglevel = merge_siglevel(c->siglevel,
+			c->remotefilesiglevel, c->remotefilesiglevel_mask);
+
+	for(i = c->repos; i; i = i->next) {
+		config_repo_t *r = i->data;
+		alpm_list_t *j;
+		SETDEFAULT(r->usage, ALPM_DB_USAGE_ALL);
+		r->siglevel = merge_siglevel(c->siglevel, r->siglevel, r->siglevel_mask);
+		for(j = r->servers; j; j = j->next) {
+			char *newurl = replace_server_vars(c, r, j->data);
+			if(newurl == NULL) {
+				return -1;
+			} else {
+				free(j->data);
+				j->data = newurl;
+			}
+		}
+	}
+
+#undef SETDEFAULT
+
+	return 0;
+}
+
+int parseconfigfile(const char *file)
+{
+	struct section_t section;
+	memset(&section, 0, sizeof(struct section_t));
+	pm_printf(ALPM_LOG_DEBUG, "config: attempting to read file %s\n", file);
+	return parse_ini(file, _parse_directive, &section);
+}
+
 /** Parse a configuration file.
  * @param file path to the config file
  * @return 0 on success, non-zero on error
@@ -1037,10 +1130,10 @@ static int _parse_directive(const char *file, int linenum, const char *name,
 int parseconfig(const char *file)
 {
 	int ret;
-	struct section_t section;
-	memset(&section, 0, sizeof(struct section_t));
-	pm_printf(ALPM_LOG_DEBUG, "config: attempting to read file %s\n", file);
-	if((ret = parse_ini(file, _parse_directive, &section))) {
+	if((ret = parseconfigfile(file))) {
+		return ret;
+	}
+	if((ret = setdefaults(config))) {
 		return ret;
 	}
 	pm_printf(ALPM_LOG_DEBUG, "config: finished parsing %s\n", file);
@@ -1052,5 +1145,3 @@ int parseconfig(const char *file)
 	config->repos = NULL;
 	return ret;
 }
-
-/* vim: set noet: */
