@@ -56,7 +56,6 @@ typedef struct __config_t {
 	unsigned short usesyslog;
 	unsigned short color;
 	unsigned short disable_dl_timeout;
-	double deltaratio;
 	char *arch;
 	char *print_format;
 	/* unfortunately, we have to keep track of paths both here and in the library
@@ -126,6 +125,8 @@ typedef struct __config_t {
 	alpm_list_t *noextract;
 	alpm_list_t *overwrite_files;
 	char *xfercommand;
+	char **xfercommand_argv;
+	size_t xfercommand_argc;
 
 	/* our connection to libalpm */
 	alpm_handle_t *handle;
