@@ -1,7 +1,7 @@
 /*
  *  deptest.c
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -32,11 +32,13 @@ int pacman_deptest(alpm_list_t *targets)
 	alpm_list_t *i;
 	alpm_list_t *deps = NULL;
 	alpm_db_t *localdb = alpm_get_localdb(config->handle);
+	alpm_list_t *pkgcache = alpm_db_get_pkgcache(localdb);
 
 	for(i = targets; i; i = alpm_list_next(i)) {
 		char *target = i->data;
 
-		if(!alpm_find_satisfier(alpm_db_get_pkgcache(localdb), target)) {
+		if(!alpm_db_get_pkg(localdb, target) &&
+				!alpm_find_satisfier(pkgcache, target)) {
 			deps = alpm_list_add(deps, target);
 		}
 	}
@@ -53,5 +55,3 @@ int pacman_deptest(alpm_list_t *targets)
 	alpm_list_free(deps);
 	return 127;
 }
-
-/* vim: set noet: */
