@@ -1,7 +1,7 @@
 /*
  *  sync.c
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -222,9 +222,7 @@ static int sync_cleancache(int level)
 					/* skip package databases within the cache directory */
 					"*.db*", "*.files*",
 					/* skip source packages within the cache directory */
-					"*.src.tar.*",
-					/* skip package deltas, we aren't smart enough to clean these yet */
-					"*.delta"
+					"*.src.tar.*"
 				};
 				size_t j;
 
@@ -521,17 +519,24 @@ static int process_pkg(alpm_pkg_t *pkg)
 
 	if(ret == -1) {
 		alpm_errno_t err = alpm_errno(config->handle);
-		if(err == ALPM_ERR_TRANS_DUP_TARGET) {
-			/* just skip duplicate targets */
-			pm_printf(ALPM_LOG_WARNING, _("skipping target: %s\n"), alpm_pkg_get_name(pkg));
-			return 0;
-		} else {
-			pm_printf(ALPM_LOG_ERROR, "'%s': %s\n", alpm_pkg_get_name(pkg),
-					alpm_strerror(err));
+		pm_printf(ALPM_LOG_ERROR, "'%s': %s\n", alpm_pkg_get_name(pkg), alpm_strerror(err));
+		return 1;
+	}
+	config->explicit_adds = alpm_list_add(config->explicit_adds, pkg);
+	return 0;
+}
+
+static int group_exists(alpm_list_t *dbs, const char *name)
+{
+	alpm_list_t *i;
+	for(i = dbs; i; i = i->next) {
+		alpm_db_t *db = i->data;
+
+		if(alpm_db_get_group(db, name)) {
 			return 1;
 		}
 	}
-	config->explicit_adds = alpm_list_add(config->explicit_adds, pkg);
+
 	return 0;
 }
 
@@ -543,6 +548,10 @@ static int process_group(alpm_list_t *dbs, const char *group, int error)
 	int count = alpm_list_count(pkgs);
 
 	if(!count) {
+		if(group_exists(dbs, group)) {
+			return 0;
+		}
+
 		pm_printf(ALPM_LOG_ERROR, _("target not found: %s\n"), group);
 		return 1;
 	}
@@ -557,9 +566,10 @@ static int process_group(alpm_list_t *dbs, const char *group, int error)
 	if(config->print == 0) {
 		char *array = malloc(count);
 		int n = 0;
-		colon_printf(_n("There is %d member in group %s:\n",
-				"There are %d members in group %s:\n", count),
-				count, group);
+		const colstr_t *colstr = &config->colstr;
+		colon_printf(_n("There is %d member in group %s%s%s:\n",
+				"There are %d members in group %s%s%s:\n", count),
+				count, colstr->groups, group, colstr->title);
 		select_display(pkgs);
 		if(!array) {
 			ret = 1;
@@ -629,7 +639,7 @@ static int process_target(const char *target, int error)
 	if(targname && targname != targstring) {
 		alpm_db_t *db;
 		const char *dbname;
-		alpm_db_usage_t usage;
+		int usage;
 
 		*targname = '\0';
 		targname++;
@@ -694,9 +704,11 @@ static int sync_trans(alpm_list_t *targets)
 	}
 
 	if(config->op_s_upgrade) {
-		colon_printf(_("Starting full system upgrade...\n"));
-		alpm_logaction(config->handle, PACMAN_CALLER_PREFIX,
-				"starting full system upgrade\n");
+		if(!config->print) {
+			colon_printf(_("Starting full system upgrade...\n"));
+			alpm_logaction(config->handle, PACMAN_CALLER_PREFIX,
+					"starting full system upgrade\n");
+		}
 		if(alpm_sync_sysupgrade(config->handle, config->op_s_upgrade >= 2) == -1) {
 			pm_printf(ALPM_LOG_ERROR, "%s\n", alpm_strerror(alpm_errno(config->handle)));
 			trans_release();
@@ -714,15 +726,16 @@ static void print_broken_dep(alpm_depmissing_t *miss)
 	alpm_pkg_t *pkg;
 	if(miss->causingpkg == NULL) {
 		/* package being installed/upgraded has unresolved dependency */
-		colon_printf(_("%s: requires %s\n"), miss->target, depstring);
+		colon_printf(_("unable to satisfy dependency '%s' required by %s\n"),
+				depstring, miss->target);
 	} else if((pkg = alpm_pkg_find(trans_add, miss->causingpkg))) {
 		/* upgrading a package breaks a local dependency */
-		colon_printf(_("%s: installing %s (%s) breaks dependency '%s'\n"),
-				miss->target, miss->causingpkg, alpm_pkg_get_version(pkg), depstring);
+		colon_printf(_("installing %s (%s) breaks dependency '%s' required by %s\n"),
+				miss->causingpkg, alpm_pkg_get_version(pkg), depstring, miss->target);
 	} else {
 		/* removing a package breaks a local dependency */
-		colon_printf(_("%s: removing %s breaks dependency '%s'\n"),
-				miss->target, miss->causingpkg, depstring);
+		colon_printf(_("removing %s breaks dependency '%s' required by %s\n"),
+				miss->causingpkg, depstring, miss->target);
 	}
 	free(depstring);
 }
@@ -809,9 +822,6 @@ int sync_prepare_execute(void)
 		        alpm_strerror(err));
 		switch(err) {
 			case ALPM_ERR_FILE_CONFLICTS:
-				if(config->flags & ALPM_TRANS_FLAG_FORCE) {
-					printf(_("unable to %s directory-file conflicts\n"), "--force");
-				}
 				for(i = data; i; i = alpm_list_next(i)) {
 					alpm_fileconflict_t *conflict = i->data;
 					switch(conflict->type) {
@@ -820,8 +830,13 @@ int sync_prepare_execute(void)
 									conflict->file, conflict->target, conflict->ctarget);
 							break;
 						case ALPM_FILECONFLICT_FILESYSTEM:
-							printf(_("%s: %s exists in filesystem\n"),
-									conflict->target, conflict->file);
+							if(conflict->ctarget[0]) {
+								printf(_("%s: %s exists in filesystem (owned by %s)\n"),
+										conflict->target, conflict->file, conflict->ctarget);
+							} else {
+								printf(_("%s: %s exists in filesystem\n"),
+										conflict->target, conflict->file);
+							}
 							break;
 					}
 					alpm_fileconflict_free(conflict);
@@ -830,7 +845,6 @@ int sync_prepare_execute(void)
 			case ALPM_ERR_PKG_INVALID:
 			case ALPM_ERR_PKG_INVALID_CHECKSUM:
 			case ALPM_ERR_PKG_INVALID_SIG:
-			case ALPM_ERR_DLT_INVALID:
 				for(i = data; i; i = alpm_list_next(i)) {
 					char *filename = i->data;
 					printf(_("%s is invalid or corrupted\n"), filename);
@@ -933,5 +947,3 @@ int pacman_sync(alpm_list_t *targets)
 
 	return sync_trans(targets);
 }
-
-/* vim: set noet: */
