@@ -1,7 +1,7 @@
 /*
  *  util.c
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -61,7 +61,7 @@ enum {
 	CELL_FREE = (1 << 3)
 };
 
-int trans_init(alpm_transflag_t flags, int check_valid)
+int trans_init(int flags, int check_valid)
 {
 	int ret;
 
@@ -103,6 +103,9 @@ int trans_release(void)
 
 int needs_root(void)
 {
+	if(config->sysroot) {
+		return 1;
+	}
 	switch(config->op) {
 		case PM_OP_DATABASE:
 			return !config->op_q_check;
@@ -148,7 +151,7 @@ int check_syncdbs(size_t need_repos, int check_valid)
 int sync_syncdbs(int level, alpm_list_t *syncs)
 {
 	alpm_list_t *i;
-	unsigned int success = 0;
+	unsigned int success = 1;
 
 	for(i = syncs; i; i = alpm_list_next(i)) {
 		alpm_db_t *db = i->data;
@@ -157,21 +160,14 @@ int sync_syncdbs(int level, alpm_list_t *syncs)
 		if(ret < 0) {
 			pm_printf(ALPM_LOG_ERROR, _("failed to update %s (%s)\n"),
 					alpm_db_get_name(db), alpm_strerror(alpm_errno(config->handle)));
+			success = 0;
 		} else if(ret == 1) {
 			printf(_(" %s is up to date\n"), alpm_db_get_name(db));
-			success++;
-		} else {
-			success++;
 		}
 	}
 
-	/* We should always succeed if at least one DB was upgraded - we may possibly
-	 * fail later with unresolved deps, but that should be rare, and would be
-	 * expected
-	 */
 	if(!success) {
-		pm_printf(ALPM_LOG_ERROR, _("failed to synchronize any databases\n"));
-		trans_init_error();
+		pm_printf(ALPM_LOG_ERROR, _("failed to synchronize all databases\n"));
 	}
 	return (success > 0);
 }
@@ -1032,16 +1028,37 @@ static char *pkg_get_location(alpm_pkg_t *pkg)
 {
 	alpm_list_t *servers;
 	char *string = NULL;
-	switch(config->op) {
-		case PM_OP_SYNC:
+	switch(alpm_pkg_get_origin(pkg)) {
+		case ALPM_PKG_FROM_SYNCDB:
+			if(alpm_pkg_download_size(pkg) == 0) {
+				/* file is already in the package cache */
+				alpm_list_t *i;
+				const char *pkgfile = alpm_pkg_get_filename(pkg);
+				char path[PATH_MAX];
+				struct stat buf;
+
+				for(i = alpm_option_get_cachedirs(config->handle); i; i = i->next) {
+					snprintf(path, PATH_MAX, "%s%s", (char *)i->data, pkgfile);
+					if(stat(path, &buf) == 0 && S_ISREG(buf.st_mode)) {
+						pm_asprintf(&string, "file://%s", path);
+						return string;
+					}
+				}
+			}
+
 			servers = alpm_db_get_servers(alpm_pkg_get_db(pkg));
 			if(servers) {
 				pm_asprintf(&string, "%s/%s", (char *)(servers->data),
 						alpm_pkg_get_filename(pkg));
 				return string;
 			}
-		case PM_OP_UPGRADE:
+
+			/* fallthrough - for theoretical serverless repos */
+
+		case ALPM_PKG_FROM_FILE:
 			return strdup(alpm_pkg_get_filename(pkg));
+
+		case ALPM_PKG_FROM_LOCALDB:
 		default:
 			pm_asprintf(&string, "%s-%s", alpm_pkg_get_name(pkg), alpm_pkg_get_version(pkg));
 			return string;
@@ -1263,8 +1280,9 @@ static void display_repo_list(const char *dbname, alpm_list_t *list,
 		unsigned short cols)
 {
 	const char *prefix = "  ";
+	const colstr_t *colstr = &config->colstr;
 
-	colon_printf(_("Repository %s\n"), dbname);
+	colon_printf(_("Repository %s%s\n"), colstr->repo, dbname);
 	list_display(prefix, list, cols);
 }
 
@@ -1281,8 +1299,9 @@ void select_display(const alpm_list_t *pkglist)
 		alpm_pkg_t *pkg = i->data;
 		alpm_db_t *db = alpm_pkg_get_db(pkg);
 
-		if(!dbname)
+		if(!dbname) {
 			dbname = alpm_db_get_name(db);
+		}
 		if(strcmp(alpm_db_get_name(db), dbname) != 0) {
 			display_repo_list(dbname, list, cols);
 			FREELIST(list);
@@ -1331,15 +1350,16 @@ static int multiselect_parse(char *array, int count, char *response)
 			break;
 		}
 		len = strtrim(starts);
-		if(len == 0)
+		if(len == 0) {
 			continue;
+		}
 
 		if(*starts == '^') {
 			starts++;
 			len--;
 			include = 0;
 		} else if(str) {
-			/* if first token is including, we unselect all targets */
+			/* if first token is including, we deselect all targets */
 			memset(array, 0, count);
 		}
 
@@ -1352,8 +1372,9 @@ static int multiselect_parse(char *array, int count, char *response)
 			}
 		}
 
-		if(parseindex(starts, &start, 1, count) != 0)
+		if(parseindex(starts, &start, 1, count) != 0) {
 			return -1;
+		}
 
 		if(!ends) {
 			array[start - 1] = include;
@@ -1477,8 +1498,9 @@ int select_question(int count)
 			size_t len = strtrim(response);
 			if(len > 0) {
 				int n;
-				if(parseindex(response, &n, 1, count) != 0)
+				if(parseindex(response, &n, 1, count) != 0) {
 					continue;
+				}
 				return (n - 1);
 			}
 		}
@@ -1750,4 +1772,25 @@ int pm_vfprintf(FILE *stream, alpm_loglevel_t level, const char *format, va_list
 	return ret;
 }
 
-/* vim: set noet: */
+char *arg_to_string(int argc, char *argv[])
+{
+	char *cl_text, *p;
+	size_t size = 0;
+	int i;
+	for(i = 0; i < argc; i++) {
+		size += strlen(argv[i]) + 1;
+	}
+	if(!size) {
+		return NULL;
+	}
+	if(!(cl_text = malloc(size))) {
+		return NULL;
+	}
+	for(p = cl_text, i = 0; i + 1 < argc; i++) {
+		strcpy(p, argv[i]);
+		p += strlen(argv[i]);
+		*p++ = ' ';
+	}
+	strcpy(p, argv[i]);
+	return cl_text;
+}
