@@ -1,4 +1,4 @@
-#! /usr/bin/python2
+#! /usr/bin/python3
 #
 #  pactest : run automated testing on the pacman binary
 #
@@ -45,7 +45,8 @@ class MultiWriter():
 # duplicate stdout/stderr to a temporary file
 class OutputSaver():
     def __init__(self):
-        self.save_file = tempfile.NamedTemporaryFile(prefix='pactest-output-')
+        self.save_file = tempfile.NamedTemporaryFile(
+                prefix='pactest-output-', mode='w')
 
     def __enter__(self):
         sys.stdout = MultiWriter(sys.stdout, self.save_file)
@@ -76,6 +77,15 @@ def create_parser():
     parser.add_option("--bindir", type = "string",
                       dest = "bindir", action = "append",
                       help = "specify location of binaries")
+    parser.add_option("--without-gpg", action = "store_true",
+                      dest = "missing_gpg", default = False,
+                      help = "skip gpg-related tests")
+    parser.add_option("--without-curl", action = "store_true",
+                      dest = "missing_curl", default = False,
+                      help = "skip downloader-related tests")
+    parser.add_option("--without-nls", action = "store_true",
+                      dest = "missing_nls", default = False,
+                      help = "skip translation-related tests")
     parser.add_option("--keep-root", action = "store_true",
                       dest = "keeproot", default = False,
                       help = "don't remove the generated pacman root filesystem")
@@ -136,6 +146,9 @@ if __name__ == "__main__":
     env.pacman["manual-confirm"] = opts.manualconfirm
     env.pacman["scriptlet-shell"] = opts.scriptletshell
     env.pacman["ldconfig"] = opts.ldconfig
+    env.config["gpg"] = not opts.missing_gpg
+    env.config["nls"] = not opts.missing_nls
+    env.config["curl"] = not opts.missing_curl
 
     try:
         for i in args:
@@ -159,6 +172,3 @@ if __name__ == "__main__":
         shutil.rmtree(root_path)
     else:
         tap.diag("pacman testing root saved: %s" % root_path)
-
-    if env.failed > 0:
-        sys.exit(1)
