@@ -1,5 +1,5 @@
 #  Copyright (c) 2006 by Aurelien Foret <orelien@chez.com>
-#  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+#  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -15,9 +15,9 @@
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
+from io import BytesIO
 import os
 import shutil
-from StringIO import StringIO
 import tarfile
 
 import pmpkg
@@ -251,11 +251,9 @@ class pmdb(object):
                     filename = os.path.join(pkg.fullname(), name)
                     info = tarfile.TarInfo(filename)
                     info.size = len(data)
-                    tar.addfile(info, StringIO(data))
+                    tar.addfile(info, BytesIO(data.encode('utf8')))
             tar.close()
             # TODO: this is a bit unnecessary considering only one test uses it
             serverpath = os.path.join(self.root, util.SYNCREPO, self.treename)
             util.mkdir(serverpath)
             shutil.copy(self.dbfile, serverpath)
-
-# vim: set ts=4 sw=4 et:
